/**
 * @file era_api.h
 * @author Copyright (c) 2016 ESET, spol. s r.o.
 */

/**
 * Some functions are deprecated. Use ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3 before including era_api.h to remove their declarations.
 */

#pragma once

#include "era_structures.h"

//--------------------------
// common
//--------------------------

/** [thread safe] Gets version of API DLL.
 * @param major [out] major part of the version number
 * @param minor [out] minor part of the version number
 * @param build [out] build part of the version number
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER = failure)
 */
uint32_t _stdcall era_GetApiVersion(uint32_t *major, uint32_t *minor, uint32_t *build);

#ifndef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
/** [thread safe] [deprecated] Gets error message for the specified error code.
 * This function is deprecated. Use era_GetErrorMessage2 instead.
 * If the provided buffer size is not sufficient, APIERROR_INSUFFICIENT_BUFFER is returned and the required buffer size is returned in BufferSize.
 * @param errorCode [in] error code
 * @param errorMessage [out] buffer for error message
 * @param bufferSize [in, out] in - size of provided buffer, out - needed buffer size if the provided buffer size is not sufficient (always including the zero character)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_INSUFFICIENT_BUFFER = failure)
 */
uint32_t _stdcall era_GetErrorMessage(uint32_t errorCode, wchar_t *errorMessage, uint32_t *bufferSize);
#endif

/** [thread safe] Gets error message for the specified error code.
 * Use era_DeleteString to deallocate the returned error message.
 * @param errorCode [in] error code
 * @param errorMessage [out] returned error message
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER = failure)
 */
uint32_t _stdcall era_GetErrorMessage2(uint32_t errorCode, wchar_t **errorMessage);

/** [thread safe] Gets last error message for the current thread.
 * Has to be called immediately after a failed API call (API call that returned other value than ERAERROR_SUCCESS) from the same thread.
 * The error message can be retrieved using the era_GetErrorMessage API call providing the value returned from the failed API call as ErrorCode. However, era_GetCurrentThreadLastErrorMessage
 * returns a specific message string filled with current context information.
 * @param message [out] last error message
 */
void _stdcall era_GetCurrentThreadLastErrorMessage(const wchar_t **message);

/** [thread safe] Deletes global data for the current thread.
 * Has to be called from all threads from which API calls have been performed.
 * Has to be called immediately before the thread ends (no API calls should be made from the thread after calling this).
 */
void _stdcall era_ThreadDataCleanup(void);

/** [NOT thread safe] Connects to ERA server.
 * Use the era_Disconnect to disconnect.
 * @param host [in] host name or IP address
 * @param port [in] port
 * @param user [in] user name (use empty string or localized administrator account name to use the administrator account)
 * @param password [in] password (can be empty string if the user has an empty password)
 * @param domainAuthentication [in] use domain authentication
 * @param useSessionCredentials [in] use current Windows session credentials (can be used only if using domain authentication)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_WINSOCK_INITIALIZATION_FAILED, APIERROR_CONNECTION_FAILED, APIERROR_CONNECTION_FAILED_INCOMPATIBLE, APIERROR_AUTHORIZATION_FAILED, APIERROR_AUTHORIZATION_USING_CURRENT_WINDOWS_SESSION_FAILED = failure)
 */
uint32_t _stdcall era_Connect(const wchar_t *host, const uint16_t port, const wchar_t *user, const wchar_t *password, const bool domainAuthentication, const bool useSessionCredentials);

/** [NOT thread safe] Disconnects from ERA server.
 * Does nothing if not connected.
 * @return error code (ERAERROR_SUCCESS = success; other = failure)
 */
uint32_t _stdcall era_Disconnect(void);

#ifndef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
/** [thread safe] [deprecated] Gets current ERA Console local path (if is installed).
 * This function is deprecated. Use era_GetLocalEraPath2 instead.
 * era_GetLocalEraPath has to be used to determine current installed ERA Console path
 * @param type [in] type of path (ERA_LOCAL_PATH_TYPE__ERA_INSTALL_DIR or ERA_LOCAL_PATH_TYPE__APPLICATION_DATA_DIR)
 * @param path [out] pointer to buffer to save path
 * @param bufferSize [in] size of buffer (including ending zero)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CANNOT_GET_LOCAL_ERA_DIR = failure)
 */
uint32_t _stdcall era_GetLocalEraPath(const Era_LocalPathType type, wchar_t *path, uint32_t bufferSize);
#endif

/** [thread safe] Gets current ERA Console local path (if is installed).
 * Use era_DeleteString to deallocate the returned path.
 * @param type [in] type of path (ERA_LOCAL_PATH_TYPE__ERA_INSTALL_DIR or ERA_LOCAL_PATH_TYPE__APPLICATION_DATA_DIR)
 * @param path [out] returned path
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER = failure)
 */
uint32_t _stdcall era_GetLocalEraPath2(const Era_LocalPathType type, wchar_t **path);

/** [thread safe] Deletes (deallocates) a string returned from an API method.
 * Use this function to delete string returned e.g. from era_GetErrorMessage2 when it is no longer needed.
 * @param string [in] string to delete
 */
void _stdcall era_DeleteString(wchar_t *string);

//--------------------------
// license
//--------------------------

/** [thread safe] Retrieves information about currently loaded license.
 * @param licenseInfo [out] structure to store information
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR = failure)
 */
uint32_t _stdcall era_GetLicenseInfo(Era_LicenseInfo **licenseInfo);

/** [thread safe] Uploads a new license to the ERA server.
 * @param fileList [in] structure for list of license files to add
 * @param replace [in] if enabled all previous license will be deleted
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_CANNOT_OPEN_FILE, APIERROR_FILE_IS_TOO_LARGE, APIERROR_NOT_ENOUGH_MEMORY, APIERROR_CANNOT_READ_FILE, APIERROR_EXPIRED_LICKEY, APIERROR_SERVER_CANNOT_CREATE_LICKEY, APIERROR_SERVER_CANNOT_WRITE_LICKEY, APIERROR_SERVER_CANNOT_UPDATE_LICKEY, APIERROR_SERVER_CANNOT_RELOAD_LICENSE = failure)
 */
uint32_t _stdcall era_AddLicense(Era_LicenseFiles *fileList, bool replace);

//--------------------------
// request_data
//--------------------------

/** [thread safe] Requests specified data from client.
 * Use era_SysinspectorParams_new to create structure for sysinspector params.
 * Used era_SysinspectorParams_delete to delete structure for sysinspector params after calling this function.
 * @param requestedTypes [in] requested types, multiple types could be requested, use any combination of values ERA_DATA_FEATURES,ERA_DATA_STATUS,ERA_DATA_SYSTEM_INFORMATION,ERA_DATA_SYSINSPECTOR
 * @param clients [in] list of clients to request data for
 * @param params [in] sysinspector specific parameters
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_PARAMETER, APIERROR_INVALID_STATUS_FROM_SERVER, APIERROR_INVALID_STATUS = failure)
 */
uint32_t _stdcall era_RequestData(const uint32_t requestedTypes, Era_ClientList *clients, Era_SysinspectorParams *params);

//--------------------------
// reports
//--------------------------

/** [thread safe] Opens a query for reading static, dashboard and generated reports.
 * Use @c era_CloseReportQuery to close every successfully opened query after reading the result.
 * @param reportQuery [in] query parameters (type of the query - static, dashboard, generated)
 * @param tableReader [out] information about columns of the result (similar to the information returned from era_GetColumnsInfo); provide this table reader to the calls to @c era_ReadReportRow and @c era_CloseReportQuery
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_QUERY_OPENING_FAILED, APIERROR_TABLE_NOT_FOUND, APIERROR_COLUMN_NOT_FOUND, APIERROR_INVALID_VALUE_IN_CONDITION, APIERROR_REPORT_QUERY_INITIALIZATION_FAILED = failure)
 */
uint32_t _stdcall era_OpenReportQuery(const Era_ReportQuery *reportQuery, Era_TableReader **tableReader);

/** [thread safe] Reads one row from a result of executing a report query.
 * Use era_OpenReportQuery to open a report query before using this function.
 * Use era_CloseReportQuery to close a report query after a reading is complete.
 * Can be used repeatedly while the returned tableRow is not NULL to read all data from the result.
 * @param type [in] type of the query - static, dashboard, generated, same as provided to @c era_OpenReportQuery
 * @param tableReader [in] table reader returned from @c era_openReportQuery
 * @param tableRow [out] values of next table row (is NULL if no more rows are available)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_QUERY_DOES_NOT_EXIST, APIERROR_INVALID_PARAMETER ,APIERROR_FIELD_GETTING_FAILED, APIERROR_REPORT_QUERY_DOES_NOT_EXIST = failure)
 */
uint32_t _stdcall era_ReadReportRow(const Era_ReportType type, const Era_TableReader *tableReader, Era_TableRow **tableRow);

/** [thread safe] Closes a report query opened using @c era_OpenReportQuery.
 * Use this to close only successfully opened report queries (when @c era_OpenReportQuery returns ERAERROR_SUCCESS).
 * Use this after reading all needed data using @c era_ReadReportRow.
 * @param type [in] type of the query - static, dashboard, generated, same as provided to @c era_OpenReportQuery
 * @param tableReader [in] table reader returned from @c era_OpenReportQuery
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_QUERY_DOES_NOT_EXIST, APIERROR_REPORT_QUERY_DOES_NOT_EXIST = failure)
 */
uint32_t _stdcall era_CloseReportQuery(const Era_ReportType type, const Era_TableReader *tableReader);

/** [thread safe] Generates a report from a static template and downloads it to a local directory.
 * The generated report does not remain stored on the server.
 * Please note that local files which already exist will be overwritten.
 * @param reportTemplateId [in] ID of the report template to use (use era_OpenReportQuery, era_ReadReportRow and era_CloseReportQuery with ERA_REPORT_TYPE__STATIC to enumerate static report templates and get available IDs)
 * @param outputDirectoryPath [in] directory to which the generated report will be downloaded (will be created if it does not exist)
 * @param outputType [in] type of the output to generate (HTML, ZIP or PDF)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_UNABLE_TO_GET_REPORT_TEMPLATE, APIERROR_REPORT_GENERATION_INITIALIZING_FAILED, APIERROR_UNABLE_TO_GET_REPORT_GENRATION_PROGRESS, APIERROR_REPORT_GENERATION_FAILED, APIERROR_REPORT_DOWNLOADING_FAILED = failure)
 */
uint32_t _stdcall era_GenerateStaticReport(const uint32_t reportTemplateId, const wchar_t *outputDirectoryPath, const Era_ReportOutput outputType);

/** [thread safe] Downloads a report stored on the server to a local directory.
 * Please note that local files which already exist will be overwritten.
 * @param reportId [in] ID of the generated report to download (use era_OpenReportQuery, era_ReadReportRow and era_CloseReportQuery with ERA_REPORT_TYPE__GENERATED to enumerate generated reports and get available IDs)
 * @param outputDirectoryPath [in] directory to which the report will be downloaded (will be created if it does not exist)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_REPORT_DOWNLOADING_FAILED = failure)
 */
uint32_t _stdcall era_DownloadServerReport(const wchar_t *reportId, const wchar_t *outputDirectoryPath);

/** [thread safe] Imports report templates from a local XML file.
 * Templates specified in the XML file which have incompatible type are skipped.
 * @param fileName [in] input XML file path
 * @param dashboard [in] type of report templates to import (if true, dashboard templates are imported, otherwise static templates are imported)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_CANNOT_LOAD_XML_FILE, APIERROR_XML_PROCESSING_ERROR, APIERROR_INVALID_STATUS_FROM_SERVER, APIERROR_ERROR_FROM_SERVER = failure)
 */
uint32_t _stdcall era_ReportTemplate_import(const wchar_t *fileName, const bool dashboard);

/** [thread safe] Exports report templates to a local XML file.
 * Templates specified in reportTemplateIDs which have incompatible type are skipped.
 * Set idCount to 0 to export all templates of the specified type.
 * @param reportTemplateIDs [in] array of report template IDs to export (can be NULL if idCount is zero)
 * @param idCount [in] count of IDs in the array
 * @param fileName [in] output XML file path
 * @param dashboard [in] type of report templates to export (if true, dashboard templates are exported, otherwise static templates are exported)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_CANNOT_OPEN_REMOTE_DATA, APIERROR_CANNOT_CREATE_FILE, APIERROR_CANNOT_READ_REMOTE_DATA, APIERROR_CANNOT_WRITE_FILE, APIERROR_INVALID_STATUS_FROM_SERVER = failure)
 */
uint32_t _stdcall era_ReportTemplate_export(const uint32_t *reportTemplateIDs, const uint32_t idCount, const wchar_t *fileName, const bool dashboard);

/** [thread safe] Creates a static or dashboard report template.
 * @param createdReportTemplateId [out] ID of the created report template
 * @param params [in] parameters for the new report template (type, name, configuration XML, description, ...)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_PARAMETER, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_FIELD_GETTING_FAILED, APIERROR_REPORT_TEMPLATE_NAME_EXISTS, APIERROR_REPORT_TEMPLATE_NAME_CONFLICT_XML_MISSING, APIERROR_FILE_DOESNT_EXIST, APIERROR_CANNOT_LOAD_XML_FILE, APIERROR_INVALID_REPORT_TEMPLATE_XML, APIERROR_XML_PROCESSING_ERROR = failure)
 */
uint32_t _stdcall era_ReportTemplate_create(uint32_t *createdReportTemplateId, const Era_ReportTemplateParams *params);

/** [thread safe] Changes parameters of a static or dashboard report template.
 * @param updatedReportTemplateId [in] ID of the updated report template
 * @param params [in] parameters for the updated report template (type, name, configuration XML, description, ...)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_PARAMETER, APIERROR_INVALID_REPORT_TEMPLATE_ID, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_FIELD_GETTING_FAILED, APIERROR_REPORT_TEMPLATE_XML_SERVER_ERROR, APIERROR_FILE_DOESNT_EXIST, APIERROR_CANNOT_LOAD_XML_FILE, APIERROR_INVALID_REPORT_TEMPLATE_XML, APIERROR_XML_PROCESSING_ERROR = failure)
 */
uint32_t _stdcall era_ReportTemplate_update(const uint32_t updatedReportTemplateId, const Era_ReportTemplateParams *params);

/** [thread safe] Deletes a report template with the specified ID.
 * @param deletedReportTemplateId [in] ID of the deleted report template
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_INVALID_REPORT_TEMPLATE_ID = failure)
 */
uint32_t _stdcall era_ReportTemplate_delete(const uint32_t deletedReportTemplateId);

//--------------------------
// server
//--------------------------

/** [thread safe] Gets information about the ERA server.
 * @param serverInfo [out] ERA server information (server name and version)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR = failure)
 */
uint32_t _stdcall era_GetServerInfo(Era_ServerInfo **serverInfo);

/** [thread safe] Retrieves a configuration XML file from the server.
 * @param fileName [in] local file name to store configuration
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_CANNOT_GET_SERVER_CONFIG, APIERROR_CANNOT_COPY_CONFIG_FILE = failure)
 */
uint32_t _stdcall era_GetServerConfiguration(const wchar_t *fileName);

/** [thread safe] Uploads a configuration XML file to the server. Only the settings present in the XML file are affected, all the other settings remain intact.
 * @param fileName [in] local path for XML configuration file to be sent
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_FILE_DOESNT_EXIST, APIERROR_CANNOT_GET_SERVER_CONFIG, APIERROR_CANNOT_OPEN_CONFIG_FILE, APIERROR_CANNOT_OPEN_SERVER_CONFIG, APIERROR_CANNOT_WRITE_CONFIG, APIERROR_UPLOAD_SERVER_CONFIG = failure)
 */
uint32_t _stdcall era_UploadServerConfiguration(const wchar_t *fileName);

/** [thread safe] Sets a particular configuration setting.
 * @param settingName [in] name of a modificating configuration setting
 * @param settingValue [in] value
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CANNOT_CREATE_CONFIG_FILE, APIERROR_CANNOT_WRITE_CONFIG, APIERROR_UPLOAD_SERVER_CONFIG, APIERROR_UNKNOWN_CFG_SETTING = failure)
 */
uint32_t _stdcall era_ServerConfigurationSetValue(const wchar_t *settingName, const wchar_t *settingValue);

/** [thread safe] Retrieves list of configuration settings available to modify by era_ServerConfigurationSetValue.
 * @param serverConfigurationList [out] structure to store a list
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER = failure)
 */
uint32_t _stdcall era_GetServerConfigurationList(Era_ServerConfigurationList **serverConfigurationList);

/** [thread safe] restarts the ERA server.
 * @param full [in] if true, the full restart will be performed (the ERA service will be also terminated)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_SERVER_RUNS_IN_APPLICATION_MODE = failure)
 * If the restart request is successfully sent, the connection to ERA server is automatically terminated.
 */
uint32_t _stdcall era_ServerRestart(bool full);

/** [thread safe] Runs the specified server actions.
 * The server actions are only scheduled to be run. This function does not wait for them to end or even to start being executed.
 * The possible actions are:
 * - ERA_RUN_NOW__CLEANUP - runs database cleanup
 * - ERA_RUN_NOW__COMPACT - runs database compacting and repairing
 * - ERA_RUN_NOW__REPLICATE - replicates to an upper server
 * - ERA_RUN_NOW__REPLICATE_WITH_MARK_ALL_CLIENTS - marks all clients for replication and replicates to an upper server
 * - ERA_RUN_NOW__UPDATE - updates ERA server
 * - ERA_RUN_NOW__UPDATE_WITH_CLEAR_CACHE - clears update cache and updates ERA server
 * - ERA_RUN_NOW__APPLY_POLICY_RULES - assigns policies to clients
 * - ERA_RUN_NOW__SYNCHRONIZE_PARAMETRIC_GROUPS - assigns clients to parametric groups
 * @param actions [in] actions to be run, use any combination of the possible actions (e.g. ERA_RUN_NOW__CLEANUP | ERA_RUN_NOW__COMPACT)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_STATUS_FROM_SERVER, APIERROR_INVALID_PARAMETER = failure)
 */
uint32_t _stdcall era_RunNow(const uint32_t actions);

//--------------------------
// tasks
//--------------------------

/** [thread safe] Creates a configuration task.
 * @param createdTaskId [out] id of created task, -1 if failed
 * @param configFilePath [in] path of a configuration XML file
 * @param clients [in] list of clients to send this task to
 * @param params [in] common task parameters (e.g. name and description)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_PARAMETER, APIERROR_TASK_INITIALIZATION_FAILED, APIERROR_TASK_CREATING_FAILED, APIERROR_UNABLE_TO_READ_FILE, APIERROR_FILE_IS_EMPTY, APIERROR_TASK_DATA_WRITING_FAILED, APIERROR_TASK_COMMIT_FAILED = failure)
 */
uint32_t _stdcall era_Task_config(uint32_t *createdTaskId, const wchar_t *configFilePath, const Era_ClientList *clients, const Era_TaskParams *params);

/** [thread safe] Creates an update task.
 * @param createdTaskId [out] id of created task, -1 if failed
 * @param windows [in] apply to Windows clients
 * @param windowsProfileName_orNull [in] profile name for Windows clients (can be NULL to not specify the profile name)
 * @param linux3 [in] apply to Linux clients with product version 3
 * @param linux [in] apply to Linux clients
 * @param mobile [in] apply to mobile clients
 * @param randomDelay_maxSeconds [in] max. delay of task execution in seconds
 * @param clients [in] list of clients to send this task to
 * @param params [in] common task parameters (e.g. name and description)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_PARAMETER, APIERROR_TASK_INITIALIZATION_FAILED, APIERROR_TASK_CREATING_FAILED, APIERROR_TASK_DATA_WRITING_FAILED, APIERROR_TASK_COMMIT_FAILED = failure)
 */
uint32_t _stdcall era_Task_update(uint32_t *createdTaskId, const bool windows, const wchar_t *windowsProfileName_orNull, const bool linux3, const bool linux, const bool mobile, const uint32_t randomDelay_maxSeconds, const Era_ClientList *clients, const Era_TaskParams *params);

/** [thread safe] Creates a scan task.
 * @param createdTaskId [out] id of created task, -1 if failed
 * @param windowsParams_orNull [in] task parameters for Windows clients (or NULL to not scan these clients)
 * @param linux3Params_orNull [in] task parameters for Linux clients with product version 3 (or NULL to not scan these clients)
 * @param linuxParams_orNull [in] task parameters for Linux clients (or NULL to not scan these clients)
 * @param mobileParams_orNull [in] task parameters for mobile clients (or NULL to not scan these clients)
 * @param randomDelay_maxSeconds [in] max. delay of task execution in seconds
 * @param clients [in] list of clients to send this task to
 * @param params [in] common task parameters (e.g. name and description)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_PARAMETER, APIERROR_TASK_INITIALIZATION_FAILED, APIERROR_TASK_CREATING_FAILED, APIERROR_TASK_DATA_WRITING_FAILED, APIERROR_TASK_COMMIT_FAILED = failure)
 */
uint32_t _stdcall era_Task_scan(uint32_t *createdTaskId, const Era_ScanParams_Windows *windowsParams_orNull, const Era_ScanParams_Linux3 *linux3Params_orNull, const Era_ScanParams_Linux *linuxParams_orNull, const Era_ScanParams_Mobile *mobileParams_orNull, const uint32_t randomDelay_maxSeconds, const Era_ClientList *clients, const Era_TaskParams *params);

//--------------------------
// policy
//--------------------------

/** [thread safe] Opens a query for reading policies in table or tree format.
 * Use @c era_ClosePolicyQuery to close every successfully opened policy query after reading the result.
 * Use table format to obtain detailed policy information.
 * Use tree format to obtain simpler policy information (only policy ID, name and primary server for each policy). May be used to build policy hierarchy tree.
 * @param policyQuery [in] query parameters (display type of the policy query - table or tree).
 * @param policyTableReader [out] information about columns of the result (similar to the information returned from @c era_GetColumnsInfo for conventional @c Era_TableReader); provide this table reader to the calls to @c era_ReadPolicyRow and @c era_ClosePolicyQuery
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_QUERY_OPENING_FAILED, APIERROR_TABLE_NOT_FOUND, APIERROR_COLUMN_NOT_FOUND, APIERROR_INVALID_VALUE_IN_CONDITION = failure)
 */
uint32_t _stdcall era_OpenPolicyQuery(const Era_PolicyQuery *policyQuery, Era_PolicyTableReader **policyTableReader);

/** [thread safe] Reads one row from a result of executing a policy query.
 * Use @c era_OpenPolicyQuery to open a policy query before using this function.
 * Use @c era_ClosePolicyQuery to close a policy query after a reading is complete.
 * Can be used repeatedly while the returned tableRow is not NULL to read all data from the result.
 * @param policyTableReader [in] policy table reader returned from @c era_openPolicyQuery
 * @param tableRow [out] values of next table row (is NULL if no more rows are available)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_POLICY_QUERY_DOES_NOT_EXIST, APIERROR_FIELD_GETTING_FAILED = failure)
 */
uint32_t _stdcall era_ReadPolicyRow(const Era_PolicyTableReader *policyTableReader, Era_TableRow **tableRow);

/** [thread safe] Closes a policy query opened using @c era_OpenPolicyQuery.
 * Use this to close only successfully opened policy queries (when @c era_OpenPolicyQuery returns ERAERROR_SUCCESS).
 * Use this after reading all needed data using @c era_ReadPolicyRow.
 * @param policyTableReader [in] policy table reader returned from @c era_OpenPolicyQuery
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_POLICY_QUERY_DOES_NOT_EXIST = failure)
 */
uint32_t _stdcall era_ClosePolicyQuery(const Era_PolicyTableReader *policyTableReader);

/** [thread safe] Creates a policy with specified parameters.
 * @param createdPolicyId [out] id of created policy
 * @param params [in] parameters for the new policy (name, configuration XML, description, ...)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_PARAMETER, APIERROR_POLICY_INVALID_PARENT, APIERROR_POLICY_CREATE_NAME_NOT_UNIQUE, APIERROR_FILE_DOESNT_EXIST, APIERROR_POLICY_CREATE_UPLOAD_XML, APIERROR_POLICY_INVALID_POLICY = failure)
 */
uint32_t _stdcall era_Policy_create(uint32_t *createdPolicyId, const Era_PolicyParams *params);

/** [thread safe] Deletes a policy with the specified ID.
 * A set of replacements has to be specified depending on the type of deleted policy and type of deleting (if the whole branch is being deleted).
 * If a replacement has not to be specified, it can be set to NULL (if the replacement is specified it will be ignored without error).
 * Each replacement can be set to some of the available states (e.g. replacing by default clients policy or replacing by a specific policy). The available states depend on the type of deleted policy, type of deleting and type of the replacement.
 * @param deletedPolicyId [in] ID of deleted policy
 * @param newParentPolicyForChildPolicies_orNull [in] replacement for the parent policy of the child policies of the policy being deleted
 * @param newPolicyForPrimaryClients_orNull [in] new policy for primary clients as a replacement for the policy being deleted
 * @param newPolicyForReplicatedClients_orNull [in] new policy for replicated clients as a replacement for the policy being deleted
 * @param newDefaultPolicyForPrimaryClients_orNull [in] new default policy for primary clients as a replacement for the policy being deleted
 * @param newDefaultPolicyForLowerServers_orNull [in] new default policy for lower servers as a replacement for the policy being deleted
 * @param deleteWholeBranch [in] if the whole branch (the specified policy and all its child policies) has to be deleted
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_POLICY_ID, APIERROR_REPLACEMENT_NEEDED, APIERROR_REPLACEMENT_NOT_NEEDED, APIERROR_REPLACEMENT_NOT_INITIALIZED, APIERROR_DEFAULT_CLIENTS_POLICY_NOT_ALLOWED, APIERROR_DEFAULT_UPPER_SERVER_POLICY_NOT_ALLOWED, APIERROR_NOT_AVAILABLE_NOT_ALLOWED, APIERROR_POLICY_CAN_NOT_BE_SET_AS_REPLACEMENT = failure)
 */
uint32_t _stdcall era_Policy_delete(const uint32_t deletedPolicyId, const Era_PolicyReplacement *newParentPolicyForChildPolicies_orNull, const Era_PolicyReplacement *newPolicyForPrimaryClients_orNull, const Era_PolicyReplacement *newPolicyForReplicatedClients_orNull, const Era_PolicyReplacement *newDefaultPolicyForPrimaryClients_orNull, const Era_PolicyReplacement *newDefaultPolicyForLowerServers_orNull, const bool deleteWholeBranch);

/** [thread safe] Updates a policy with the specified parameters.
 * @see era_Policy_delete for more information about policy replacements.
 * @param policyId [in] ID of the updated policy
 * @param params [in] parameters for the updated policy (name, configuration XML, description, ...)
 * @param newPolicyForReplicatedClients_orNull [in] new policy for replicated clients as a replacement for the policy being updated
 * @param newDefaultPolicyForLowerServers_orNull [in] new default policy for lower servers as a replacement for the policy being updated
 * @param newDefaultPolicyForPrimaryClients_orNull [in] new default policy for primary clients as a replacement for the policy being updated
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_PARAMETER, APIERROR_CANNOT_ASSIGN_DEFAULT_PARENT_POLICY, APIERROR_FILE_DOESNT_EXIST, APIERROR_POLICY_UPDATE_UPLOAD_XML, APIERROR_POLICY_INVALID_POLICY, APIERROR_REPLACEMENT_NEEDED, APIERROR_REPLACEMENT_NOT_INITIALIZED, APIERROR_DEFAULT_CLIENTS_POLICY_NOT_ALLOWED, APIERROR_DEFAULT_UPPER_SERVER_POLICY_NOT_ALLOWED, APIERROR_NOT_AVAILABLE_NOT_ALLOWED, APIERROR_INVALID_POLICY_ID, APIERROR_POLICY_CAN_NOT_BE_SET_AS_REPLACEMENT, APIERROR_POLICY_UPDATE_NAME_NOT_UNIQUE, APIERROR_POLICY_CANNOT_UPDATE = failure)
 */
uint32_t _stdcall era_Policy_update(const uint32_t policyId, const Era_PolicyParams *params, const Era_PolicyReplacement *newPolicyForReplicatedClients_orNull, const Era_PolicyReplacement *newDefaultPolicyForLowerServers_orNull, const Era_PolicyReplacement *newDefaultPolicyForPrimaryClients_orNull);

/** [thread safe] Assigns policy to clients.
 * This function also validates input: It is not possible to assign any policy to any client. If the client list contains replicated clients, the policy must be down-replicable. Policy from lower servers can not be assigned.
 * @param policyId [in] ID of assigned policy. 0 for default clients policy
 * @param clients [in] structure with list of clients
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_PARAMETER, APIERROR_CANNOT_ASSIGN_DEFAULT_PARENT_POLICY, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_FIELD_GETTING_FAILED, APIERROR_CANNOT_READ_POLICY_INFO, APIERROR_CANNOT_ASSIGN_INACTIVE_POLICY, APIERROR_ONLYLOCAL_POLICY_COULD_BE_ASSIGNED = failure)
 */
uint32_t _stdcall era_Policy_assign(const uint32_t policyId, const Era_ClientList *clients);

/** [thread safe] Imports policies from an XML file.
 * @param fileName [in] XML file path
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_CANNOT_LOAD_XML_FILE, APIERROR_XML_PROCESSING_ERROR, APIERROR_INVALID_STATUS, APIERROR_ERROR_FROM_SERVER = failure)
 */
uint32_t _stdcall era_Policy_import(const wchar_t *fileName);

/** [thread safe] Exports specified policy to an XML file.
 * @param policyId [in] ID of assigned policy. 0 for default clients policy
 * @param exportAll [in] export all policies
 * @param tree [in] export policy tree starting with specified policy. not applicable when exporting all policies
 * @param fileName [in] output XML file path 
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_CANNOT_LOAD_XML_FILE, APIERROR_XML_PROCESSING_ERROR, APIERROR_INVALID_STATUS_FROM_SERVER = failure)
  */
uint32_t _stdcall era_Policy_export(const uint32_t policyId, const bool exportAll, const bool tree, const wchar_t *fileName);

//--------------------------
// policy rules
//--------------------------

/** [thread safe] Opens a query for reading policy rules.
 * Use @c era_ClosePolicyRulesQuery to close every successfully opened policy rules query after reading the result.
 * @param tableReader [out] information about columns of the result (similar to the information returned from era_GetColumnsInfo); provide this table reader to the calls to @c era_ReadPolicyRulesRow and @c era_ClosePolicyRulesQuery
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_QUERY_OPENING_FAILED, APIERROR_TABLE_NOT_FOUND, APIERROR_COLUMN_NOT_FOUND, APIERROR_INVALID_VALUE_IN_CONDITION = failure)
 */
uint32_t _stdcall era_OpenPolicyRulesQuery(Era_TableReader **tableReader);

/** [thread safe] Reads one row from a result of executing a policy rules query.
 * Use @c era_OpenPolicyRulesQuery to open a policy rules query before using this function.
 * Use @c era_ClosePolicyRulesQuery to close a policy rules query after a reading is complete.
 * Can be used repeatedly while the returned tableRow is not NULL to read all data from the result.
 * @param tableReader [in] table reader returned from @c era_openPolicyRulesQuery
 * @param tableRow [out] values of next table row (is NULL if no more rows are available)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_QUERY_DOES_NOT_EXIST, APIERROR_FIELD_GETTING_FAILED = failure)
 */
uint32_t _stdcall era_ReadPolicyRulesRow(const Era_TableReader *tableReader, Era_TableRow **tableRow);

/** [thread safe] Closes a policy rules query opened using @c era_OpenPolicyRulesQuery.
 * Use this to close only successfully opened policy rules queries (when @c era_OpenPolicyRulesQuery returns ERAERROR_SUCCESS).
 * Use this after reading all needed data using @c era_ReadPolicyRulesRow.
 * @param tableReader [in] table reader returned from @c era_OpenPolicyRulesQuery
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_QUERY_DOES_NOT_EXIST = failure)
 */
uint32_t _stdcall era_ClosePolicyRulesQuery(const Era_TableReader *tableReader);

/** [NOT thread safe] Creates new policy rule.
 * @param createdRuleId [out] id of created task, -1 if failed
 * @param ruleParams [in] parameters for policy rule creation
  * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_RULE_NAME_EXISTS, APIERROR_CANNOT_READ_POLICY_INFO, APIERROR_ONLYLOCAL_POLICY_COULD_BE_USED, APIERROR_CANNOT_LOAD_XML_FILE, APIERROR_XML_PROCESSING_ERROR, APIERROR_RULE_NOT_FOUND_IN_XML, APIERROR_NO_RULE_CREATED_ON_SERVER, APIERROR_INVALID_STATUS = failure)
 */
uint32_t _stdcall era_CreatePolicyRule(uint32_t *createdRuleId, const Era_PolicyRuleParams *ruleParams);

/** [thread safe] Deletes a policy rule with the specified ID.
 * @param deletedRuleId [in] ID of the deleted policy rule
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_POLICY_RULE_ID, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_FIELD_GETTING_FAILED = failure)
 */
uint32_t _stdcall era_PolicyRule_delete(const uint32_t deletedRuleId);

/** [thread safe] Imports policy rules from an XML file.
 * @param fileName [in] XML file path
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_CANNOT_LOAD_XML_FILE, APIERROR_XML_PROCESSING_ERROR, APIERROR_INVALID_STATUS, APIERROR_ERROR_FROM_SERVER = failure)
 */
uint32_t _stdcall era_PolicyRule_import(const wchar_t *fileName);

/** [thread safe] Exports policy rules to an XML file.
 * @param ruleIDs [in] list of rule IDs (if the parameter idCount is zero, can be NULL)
 * @param idCount [in] count of IDs given by the ruleIDs parameter, if zero the all rules will be exported
 * @param fileName [in] output XML file path
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_INVALID_POLICY_RULE_ID, APIERROR_CANNOT_OPEN_REMOTE_DATA, APIERROR_CANNOT_CREATE_FILE, APIERROR_CANNOT_READ_REMOTE_DATA, APIERROR_CANNOT_WRITE_FILE = failure)
 */
uint32_t _stdcall era_PolicyRule_export(uint32_t* ruleIDs, const uint32_t idCount, const wchar_t *fileName);

/** [NOT thread safe] Changes parameters or configuration of a policy rule.
 * Only the changed parameters in the ruleData structure (with other values than default) will be applied.
 * @param ruleId [in] policy rule to change
 * @param ruleParams [in] parameters for policy rule change
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_ONLY_LOCAL_RULE_CAN_BE_CHANGED, APIERROR_ONLYLOCAL_POLICY_COULD_BE_USED, APIERROR_CANNOT_LOAD_XML_FILE, APIERROR_XML_PROCESSING_ERROR, APIERROR_INVALID_STATUS, APIERROR_INVALID_POLICY_RULE_ID, APIERROR_FIELD_GETTING_FAILED, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_INVALID_POLICY_ID, APIERROR_INVALID_POLICY_RULE_ID  = failure)
 */
uint32_t _stdcall era_PolicyRule_update(const uint32_t ruleId, const Era_PolicyRuleParams *ruleParams);

//--------------------------
// group
//--------------------------

/** [thread safe] Opens a query for reading groups in table or tree format.
 * Use @c era_CloseGroupQuery to close every successfully opened group query after reading the result.
 * Use table format to obtain detailed group information.
 * Use tree format to obtain enough information to be used to build group hierarchy tree (at the moment, unlike policy queries, there is no difference between the two types).
 * @param groupQuery [in] query parameters (display type of the group query - table or tree).
 * @param groupTableReader [out] information about columns of the result (similar to the information returned from @c era_GetColumnsInfo for conventional @c Era_TableReader); provide this table reader to the calls to @c era_ReadGroupRow and @c era_CloseGroupQuery
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_TABLE_NOT_FOUND, APIERROR_COLUMN_NOT_FOUND = failure)
 */
uint32_t _stdcall era_OpenGroupQuery(const Era_GroupQuery *groupQuery, Era_GroupTableReader **groupTableReader);

/** [thread safe] Reads one row from a result of executing a group query.
 * Use @c era_OpenGroupQuery to open a group query before using this function.
 * Use @c era_CloseGroupQuery to close a group query after a reading is complete.
 * Can be used repeatedly while the returned tableRow is not NULL to read all data from the result.
 * @param groupTableReader [in] group table reader returned from @c era_openGroupQuery
 * @param tableRow [out] values of next table row (is NULL if no more rows are available)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_GROUP_QUERY_DOES_NOT_EXIST, APIERROR_FIELD_GETTING_FAILED = failure)
 */
uint32_t _stdcall era_ReadGroupRow(const Era_GroupTableReader *groupTableReader, Era_TableRow **tableRow);

/** [thread safe] Closes a group query opened using @c era_OpenGroupQuery.
 * Use this to close only successfully opened group queries (when @c era_OpenGroupQuery returns ERAERROR_SUCCESS).
 * Use this after reading all needed data using @c era_ReadGroupRow.
 * @param groupTableReader [in] group table reader returned from @c era_OpenGroupQuery
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_GROUP_QUERY_DOES_NOT_EXIST = failure)
 */
uint32_t _stdcall era_CloseGroupQuery(const Era_GroupTableReader *groupTableReader);

/** [thread safe] Creates a group with the specified parameters.
 * @param createdGroupId [out] id of created group
 * @param params [in] parameters for the new group (name, description, parent ID, ...)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_PARAMETER, APIERROR_FILE_DOESNT_EXIST, APIERROR_GROUPPARENTNEXST, APIERROR_GROUP_NAME_OR_DESC_CANNOT_CONTAIN_CHARS, APIERROR_LGTOOLONGNAME, APIERROR_LGTOOLONGDESC, APIERROR_GROUP_PARAM_NOT_APPLICABLE, APIERROR_GROUP_PARAM_MISSING, APIERROR_CANNOT_LOAD_XML_FILE, APIERROR_XML_PROCESSING_ERROR, APIERROR_INVALID_GROUP_RULE_XML = failure)
 */
uint32_t _stdcall era_Group_create(uint32_t *createdGroupId, const Era_GroupParams *params);

/** [thread safe] Updates a group with the specified parameters.
 * @param updatedGroupId [out] id of the updated group
 * @param params [in] new parameters for the updated group (name, description, parent ID, ...)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_PARAMETER, APIERROR_INVALID_GROUP_ID, APIERROR_GROUP_PARAM_NOT_APPLICABLE_XML, APIERROR_GROUP_PARAM_STICKY_UNCHANGABLE, APIERROR_GROUP_NAME_OR_DESC_CANNOT_CONTAIN_CHARS, APIERROR_LGTOOLONGNAME, APIERROR_LGTOOLONGDESC, APIERROR_PARAMGROUP_EXISTS, APIERROR_STATICGROUP_EXISTS, APIERROR_GROUPPARENTNEXST, APIERROR_INVALID_GROUP_PARENT_TYPE, APIERROR_INVALID_GROUP_PARENT, APIERROR_FILE_DOESNT_EXIST, APIERROR_CANNOT_LOAD_XML_FILE, APIERROR_INVALID_GROUP_RULE_XML, APIERROR_XML_PROCESSING_ERROR = failure)
 */
uint32_t _stdcall era_Group_update(const uint32_t updatedGroupId, const Era_GroupParams *params);

/** [thread safe] Deletes a group with the specified ID and all its subgroups.
 * @param deletedGroupId [in] ID of the deleted group
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_GROUP_ID = failure)
 */
uint32_t _stdcall era_Group_delete(const uint32_t deletedGroupId);

/** [thread safe] Assigns clients to static group.
 * @param groupId [in] ID of assigned group
 * @param clients [in] structure with list of assgined clients
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_INVALID_GROUP_ID, APIERROR_INVALID_GROUP_TYPE = failure)
 */
uint32_t _stdcall era_Group_assign(const uint32_t groupId, const Era_ClientList *clients);

/** [thread safe] Removes clients from a static group.
 * @param groupId [in] ID of a group to remove client(s) from
 * @param clients [in] structure with list of removed clients
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_INVALID_GROUP_ID, APIERROR_INVALID_GROUP_TYPE_ASSIGN, APIERROR_QUERY_INITIALIZATION_FAILED = failure)
 */
uint32_t _stdcall era_Group_remove(const uint32_t groupId, const Era_ClientList *clients);

/** [thread safe] Imports static or parametric groups from a local XML file.
 * If a group with the same path already exists, it will be changed (e.g. specified client relations will be added to the existing group).
 * @param parentId [in] ID of the parent group - the groups will be imported under this group in the group tree (use 0 to import under the group tree root)
 * @param fileName [in] input XML file path
 * @param parametric [in] type of groups to import (if true, parametric groups are imported, otherwise static groups are imported)
 * @param importClientRelations [in] imports client relations of static groups, too
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_CANNOT_LOAD_XML_FILE, APIERROR_XML_PROCESSING_ERROR, APIERROR_FIELD_GETTING_FAILED, APIERROR_INVALID_STATUS_FROM_SERVER = failure)
 */
uint32_t _stdcall era_Group_import(const uint32_t parentId, const wchar_t *fileName, const bool parametric, const bool importClientRelations);

/** [thread safe] Exports static or parametric groups to a local XML file.
 * @param groupId [in] ID of the group or ID of the root group of the subtree to export (use 0 and set tree to TRUE to export the whole group tree)
 * @param tree [in] if whole subtree (with groupId as root) has to be exported
 * @param fileName [in] output XML file path
 * @param parametric [in] type of groups to export (if true, parametric groups are exported, otherwise static groups are exported)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_XML_PROCESSING_ERROR, APIERROR_FIELD_GETTING_FAILED, APIERROR_CANNOT_WRITE_FILE, APIERROR_INVALID_STATUS_FROM_SERVER = failure)
 */
uint32_t _stdcall era_Group_export(const uint32_t groupId, const bool tree, const wchar_t *fileName, const bool parametric);

//--------------------------
// database
//--------------------------

/** [thread safe] Gets a list of available table names.
 * Use a name from this list to specify a table in era_GetColumnsInfo and era_OpenQuery.
 * @param tablesInfo [out] list of available tables
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER = failure)
 */
uint32_t _stdcall era_GetTablesInfo(Era_TablesInfo **tablesInfo);

/** [thread safe] Gets a list of columns from the specified table.
 * A column information includes column name (header), translatable column name (pretty header) and data type information.
 * @param tableName [in] table name (use era_GetTablesInfo to get a list of available table names)
 * @param columnsInfo [out] information about columns for the specified table
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER = failure)
 */
uint32_t _stdcall era_GetColumnsInfo(const wchar_t *tableName, Era_TableReader **columnsInfo);

/** [thread safe] Gets row count of a result of the specified query.
 * The count equals to the count of rows that could be read using era_ReadRow for the same query.
 * Query parameters skip and limit cannot be used.
 * @param query [in] query parameters (e.g. table name, columns, conditions) - use era_GetTablesInfo to get available table names and era_GetColumnsInfo to get available column names (the non-pretty column names are used here)
 * @param count [out] row count
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_QUERY_OPENING_FAILED, APIERROR_TABLE_NOT_FOUND, APIERROR_COLUMN_NOT_FOUND, APIERROR_INVALID_VALUE_IN_CONDITION, APIERROR_COLUMN_FILTERING_ONLY, APIERROR_UNSUPPORTED_OPERATOR = failure)
 */
uint32_t _stdcall era_GetCount(const Era_Query *query, uint32_t *count);

/** [thread safe] Opens (executes) a database query.
 * After opening, use era_ReadRow to retrieve next row from the result.
 * Use era_CloseQuery to close every successfully opened query after reading the result
 * @param query [in] query parameters (e.g. table name, columns, conditions) - use era_GetTablesInfo to get available table names and era_GetColumnsInfo to get available column names (the non-pretty column names are used here)
 * @param tableReader [out] contains information about the columns of the result (similar to the information returned from era_GetColumnsInfo); provide this table reader to the calls to era_ReadRow and era_CloseQuery
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_QUERY_OPENING_FAILED, APIERROR_TABLE_NOT_FOUND, APIERROR_COLUMN_NOT_FOUND, APIERROR_INVALID_VALUE_IN_CONDITION, APIERROR_COLUMN_FILTERING_ONLY, APIERROR_COLUMN_ORDER_BY_MORE_THAN_ONCE, APIERROR_UNSUPPORTED_OPERATOR = failure)
 */
uint32_t _stdcall era_OpenQuery(const Era_Query *query, Era_TableReader **tableReader);

/** [thread safe] Reads one row from a result of executing a query.
 * Use era_OpenQuery to open a query before using this function.
 * Use era_CloseQuery to close a query after a reading is complete.
 * Can be used repeatedly while the returned tableRow is not NULL to read all data from the result.
 * @param tableReader [in] table reader returned from era_OpenQuery
 * @param tableRow [out] contains values of next table row (is NULL if no more rows are available)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_QUERY_DOES_NOT_EXIST, APIERROR_FIELD_GETTING_FAILED = failure)
 */
uint32_t _stdcall era_ReadRow(Era_TableReader *tableReader, Era_TableRow **tableRow);

/** [thread safe] Closes a database query opened using era_OpenQuery.
 * Use this to close only successfully opened queries (when era_OpenQuery returns ERAERROR_SUCCESS).
 * Use this after reading all needed data using era_ReadRow.
 * @param tableReader [in] table reader returned from era_OpenQuery
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_QUERY_DOES_NOT_EXIST = failure)
 */
uint32_t _stdcall era_CloseQuery(const Era_TableReader *tableReader);

//--------------------------
// client
//--------------------------

/** [thread safe] Renames a client on the server.
 * If such client name exist on the server, the client is not renamed an error is displayed.
 * @param clientId [in] ID of the client to rename
 * @param name [in] new name for client with id @c clientId
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_CLIENT_NAME_DUPLICATE = failure)
 */
uint32_t _stdcall era_Client_rename(const uint32_t clientId, const wchar_t *name);

/** [thread safe] Sets a comment for a client.
 * @param clientId [in] ID of the client to set the comment
 * @param comment [in] comment to set to client with id @c clientId
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_CLIENT_COMMENT_NOCHANGE = failure)
 */
uint32_t _stdcall era_Client_comment(const uint32_t clientId, const wchar_t *comment);

/** [thread safe] Sets or resets a new flag for clients.
 * @param clients [in] list of clients to set or reset the new user flag to
 * @param set [in] true to set the new flag, false to reset it
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_EMPTY_CLIENTLIST, APIERROR_CLIENT_NEW_NOCHANGE = failure)
 */
uint32_t _stdcall era_Client_new(const Era_ClientList *clients, const bool set);

/** [thread safe] Deletes clients from server.
 * @param clients [in] list of clients to set or reset the new user flag to. Deleting all clients using era_ClientList_SetAllClients is not supported.
 * @param noWait [in] do not wait for the result of processing the request on the server.
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_EMPTY_CLIENTLIST, APIERROR_NOT_ENOUGH_MEMORY, APIERROR_INTERNAL_ERROR, APIERROR_INVALID_STATUS_FROM_SERVER, APIERROR_ERROR_FROM_SERVER = failure)
 */
uint32_t _stdcall era_Client_delete(const Era_ClientList *clients, const bool noWait);

/** [thread safe] Sets or resets a roaming user flag for clients.
 * @param clients [in] list of clients to set or reset the roaming user flag to
 * @param set [in] true to set the roaming user flag, false to reset it
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_EMPTY_CLIENTLIST, APIERROR_CLIENT_ROAMING_NOCHANGE = failure)
 */
uint32_t _stdcall era_Client_roaming(const Era_ClientList *clients, const bool set);

/** [thread safe] Downloads specified set of information from the particular client or about particular policy to the local file.
 * This function also validates input, e.g. invalid dataType argument or NULL pointer.
 * @param dataType [in] type of information to be downloaded (ERA_DATA_CONFIGURATION, ERA_DATA_FEATURES, ERA_DATA_STATUS, ERA_DATA_SYSTEM_INFORMATION, ERA_DATA_SYSINSPECTOR, ERA_DATA_POLICY, ERA_DATA_POLICY_MERGED, ERA_DATA_POLICY_OVERRIDE, ERA_DATA_POLICY_NONOVERRIDE)
 * @param dataId [in] ID of the data entity (client or policy) to download data from
 * @param localFile [in] destination filename, where the downloaded data will be stored
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_PARAMETER, APIERROR_CANNOT_OPEN_REMOTE_DATA, APIERROR_CANNOT_CREATE_FILE, APIERROR_CANNOT_READ_REMOTE_DATA, APIERROR_CANNOT_WRITE_FILE, APIERROR_CANNOT_OPEN_XML_DATA, APIERROR_CANNOT_READ_XML_DATA, APIERROR_POLICY_INVALID_POLICY, APIERROR_POLICY_DATA_NO_UPPER, APIERROR_POLICY_DATA_ONLY_UPPER, APIERROR_POLICY_NO_DATA, APIERROR_POLICY_GENERATE, APIERROR_POLICY_CONVERT = failure)
 */
uint32_t _stdcall era_GetData(const uint32_t dataType, const uint32_t dataId, const wchar_t *localFile);

//--------------------------
// password
//--------------------------

/** [thread safe] Changes server security password.
 * The user must enter old password or administrator password to set a new one
 * @param passwordType [in] type of password to set
 * @param oldPassword [in] old password. also administrator password could be used
 * @param newPassword [in] new password
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_OLD_PASSWORD_INCORRECT, APIERROR_INVALID_PARAMETER = failure)
 */
uint32_t _stdcall era_ChangePassword(const Era_PasswordType passwordType, const wchar_t *oldPassword, const wchar_t *newPassword);

/** [thread safe] Encrypts given value for use it in configurations.
 * Depending of the value type, the following types of encryption can be used:
 * ERA server passwords (replication, client, installer, users) should be encrypted using ERA_PASSWORD_ENCRYPTIONTYPE__SERVER. 
 * Other passwords (SMTP, update,..) should be encrypted using ERA_PASSWORD_ENCRYPTIONTYPE__OTHER
 * Passwords hashes (lock password) should be encrypted using ERA_PASSWORD_ENCRYPTIONTYPE__HASH
 * Big numeric values (64-bit integers) should be encrypted using ERA_PASSWORD_ENCRYPTIONTYPE__NUM
 * @param encryptionType [in] type of encryption
 * @param password [in] password for encryption
 * @param encryptedPassword [out] Encrypted password
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_INTERNAL_ERROR = failure)
 */
uint32_t _stdcall era_EncryptPassword(const Era_PasswordEncryptionType encryptionType, const wchar_t* password, Era_EncryptedPassword **encryptedPassword);

//--------------------------
// scan log
//--------------------------

/** [thread safe] Opens specified scan log.
 * Use era_CloseScanLog to close opened handle and release memory allocated by this function
 * Use era_GetScanLogRow to get current line of an opened scan log
 * @param logId [in] ID of the scan log to get
 * @param scanLog [out] pointer to handle holding structures and data of required scan log
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CANNOT_READ_REMOTE_DATA = failure)
 */
uint32_t _stdcall era_OpenScanLog(const uint64_t logId, Era_ScanLog **scanLog);


/** [thread safe] Retrieves one record of an opened scan log.
 * @param scanLog [in] pointer to handle of an opened scan log
 * @param prettyLevel [in] defines if level text has to be returned in a pretty readable form
 * @param level [out] pointer to log level of log line
 * @param line [out] pointer to log line (value is valid until next call of this function or era_CloseScanLog)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CANNOT_READ_REMOTE_DATA, APIERROR_NO_MORE_DATA) = failure)
 */
uint32_t _stdcall era_GetScanLogRow(Era_ScanLog *scanLog, bool prettyLevel, const wchar_t **level, const wchar_t **line);


/** [thread safe] Closes open handles and release memory allocated by calling function.
 * @param scanLog [in] scan log handle to close
 */
void _stdcall era_CloseScanLog(Era_ScanLog *scanLog);

//--------------------------
// log forwarding
//--------------------------

/** [thread safe] Gets information about log forwarding.
 * This function validates input, e.g. invalid dataType argument or NULL pointer.
 * Use era_LogForwardInfo_delete to release structure created by calling this function.
 * @param logType [in] log type of information to be downloaded (event, threat, firewall, hips, antispam, greylist, scan, mobile, device_control, web_control), empty string for all log types
 * @param logForwardInfo [out] logForwardInfo structure to store information
 * @param prettyLevel [in] defines if level text has to be returned in a pretty readable form
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_CANNOT_READ_REMOTE_DATA = failure)
 */
uint32_t _stdcall era_GetLogForwardInfo(const wchar_t *logType, Era_LogForwardInfo **logForwardInfo, bool prettyLevel);

/** [thread safe] Sets parameters for log forwarding.
 * This function validates input, e.g. invalid dataType argument or NULL pointer.
 * @param logType [in] log type to be set (event, threat, firewall, hips, antispam, greylist, scan, mobile, device_control, web_control)
 * @param enable [in] true, if specified log forwarding has to be active
 * @param logLevel [in] log level value
 * @param severity [in] syslog severity value
 * @param facility [in] syslog facility (value from 0 upto 20)
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_CONNECTION_ERROR, APIERROR_INVALID_PARAMETER, APIERROR_LOGLEVEL_NOT_SUPPORTED, APIERROR_QUERY_INITIALIZATION_FAILED, APIERROR_CANNOT_READ_REMOTE_DATA, APIERROR_NOTIFICATION_TABLE_CORRUPTED, APIERROR_CANNOT_WRITE_XML_DATA, APIERROR_LOGFORWARD_UPDATE_FAILED = failure)
 */
uint32_t _stdcall era_SetLogForward(const wchar_t *logType, bool enable, const wchar_t *logLevel, const wchar_t *severity, const wchar_t *facility);

//--------------------------
// clear info
//--------------------------

/** [thread safe] Clears specified information for clients.
 * @param clearedInfo [in] cleared information types, multiple types could be requested, use any combination of values ERA_INFO_CLEAR_LAST_THREAT,ERA_INFO_CLEAR_LAST_EVENT,ERA_INFO_CLEAR_LAST_FIREWALL,ERA_INFO_CLEAR_LAST_SCANS,ERA_INFO_CLEAR_CUSTOM_INFO
 * @param clients [in] list of clients to clear information for
 * @return error code (ERAERROR_SUCCESS = success; APIERROR_INVALID_PARAMETER, APIERROR_CONNECTION_ERROR, APIERROR_EMPTY_CLIENTLIST, APIERROR_CLEARINFO_NOCHANGE = failure)
 */
uint32_t _stdcall era_ClearInfo(const uint32_t clearedInfo, Era_ClientList *clients);
