#pragma once

#include <functional>

template<typename DiscardedReturnValue>
class scope_exit_discard_return {

public:
	typedef std::function<DiscardedReturnValue()> FunctionType;
	explicit scope_exit_discard_return(FunctionType f)
	:
		f(f)
	{
	}

	void disable() {
		f=NULL;
	}

    ~scope_exit_discard_return() {
		if (f != NULL) {
			(void)f();
		}
	}

private:
	scope_exit_discard_return(const scope_exit_discard_return<DiscardedReturnValue> &other);

private:
    FunctionType f;

};

typedef scope_exit_discard_return<void> scope_exit;
