/**
 * @file ConsoleKeyboardInput.h
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#pragma once

#include "autocomplete/Autocomplete.h"

class ScriptInterpreter;

class ConsoleKeyboardInput
{
public:
	ConsoleKeyboardInput(ScriptInterpreter &scriptInterpreter, bool inkognito = false);
	void GetInput();
	const std::wstring &GetLine() { return line; }
	void SetInkognitoMode(bool inkongnito) { inkognitoMode = inkongnito; }

private:
	void AutoComplete(bool remove);

	void AddTypedChar(WCHAR ch);
	void RedrawInput();
	void MoveCursor(int offset);

	void GetHistory(bool prior);
	void ReplaceInput(const std::wstring &newInput);
	void ToggleInsert();
	void DeleteChar(int pos);

	COORD GetCoordForPosition(int pos);


	void WriteText(HANDLE hConsoleOutput, LPCWSTR lpCharacter, DWORD nLength, COORD dwWriteCoord, LPDWORD lpNumberOfCharsWritten);
	void WriteChar(HANDLE hConsoleOutput, WCHAR cCharacter, DWORD nLength, COORD dwWriteCoord, LPDWORD lpNumberOfCharsWritten);

	void ApplyCurrentAttributes(HANDLE hConsoleOutput, DWORD nLength, COORD dwWriteCoord);

private:
	HANDLE hStdin, hStdout;
	std::wstring line; // line content
	std::wstring lineB4History; // line before using history
	bool historyActive;
	int cursorPos; // line index
	COORD lineStartPosition; // line start position in console buffer
	COORD consoleBufferSize;
	bool overwrite;
	DWORD normalCursorSize;
	bool inkognitoMode;



	// autocomplete
	Autocomplete autoComplete;
	COORD cursorAutoCompleteStartPosition; // position in console buffer
	std::wstring lastAutoCompleteSuggestion; // so we can remove it if user presses another tab

};

