/**
 * @file ErrorMessages.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "ErrorMessages.h"

#include <Common/era_api/err_codes.h>
#include <Common/era_api/err_common.h>
#include <Common/era_api/err_lookup.h>
#include "resource.h"

#define ADD_ERROR(err) gErrorTable.Add(err, IDS_TXPLN_##err##, IDS_TXFMT_##err##);

void LoadErrorMessages(HMODULE h)
{
	gErrorTable.Init(h);
	gErrorTable.Add(CMDERROR_INVALID_PARAMETER, IDS_TXPLN_CMDERROR_INVALID_PARAMETER, IDS_TXFMT_CMDERROR_INVALID_PARAMETER);
	gErrorTable.Add(CMDERROR_INVALID_COMMAND_SYNTAX, IDS_TXPLN_CMDERROR_INVALID_COMMAND_SYNTAX, IDS_TXFMT_CMDERROR_INVALID_COMMAND_SYNTAX);
	gErrorTable.Add(CMDERROR_UNEXPECTED_COMMA, IDS_TXPLN_CMDERROR_UNEXPECTED_COMMA, IDS_TXPLN_CMDERROR_UNEXPECTED_COMMA);
	gErrorTable.Add(CMDERROR_UNKNOWN_COMMAND, IDS_TXPLN_CMDERROR_UNKNOWN_COMMAND, IDS_TXFMT_CMDERROR_UNKNOWN_COMMAND);



	// *-> SECTION 1
	gErrorTable.Add(CMDERROR_INVALID_PARAMETER, IDS_TXPLN_CMDERROR_INVALID_PARAMETER, IDS_TXFMT_CMDERROR_INVALID_PARAMETER);
	// <-* SECTION 1



	// *-> SECTION 2
	gErrorTable.Add(CMDERROR_UNEXPECTED_INPUT_END, IDS_TXPLN_CMDERROR_UNEXPECTED_INPUT_END, IDS_TXPLN_CMDERROR_UNEXPECTED_INPUT_END);
	gErrorTable.Add(CMDERROR_UNEXPECTED_CONN_PARAM, IDS_TXPLN_CMDERROR_UNEXPECTED_CONN_PARAM, IDS_TXFMT_CMDERROR_UNEXPECTED_CONN_PARAM);
	gErrorTable.Add(CMDERROR_INVALID_PORT, IDS_TXPLN_CMDERROR_INVALID_PORT, IDS_TXFMT_CMDERROR_INVALID_PORT);
	gErrorTable.Add(CMDERROR_EMPTY_CONN_PARAM, IDS_TXPLN_CMDERROR_EMPTY_CONN_PARAM, IDS_TXPLN_CMDERROR_EMPTY_CONN_PARAM);
	gErrorTable.Add(CMDERROR_INVALID_CONN_PARAM, IDS_TXPLN_CMDERROR_INVALID_CONN_PARAM, IDS_TXFMT_CMDERROR_INVALID_CONN_PARAM);
	gErrorTable.Add(CMDERROR_CONFLICT_OPTIONS_TWO, IDS_TXPLN_CMDERROR_CONFLICT_OPTIONS_TWO, IDS_TXFMT_CMDERROR_CONFLICT_OPTIONS_TWO);
	gErrorTable.Add(CMDERROR_FILE_OPEN_ERROR, IDS_TXPLN_CMDERROR_FILE_OPEN_ERROR, IDS_TXFMT_CMDERROR_FILE_OPEN_ERROR);
	gErrorTable.Add(CMDERROR_ENCODING_CONVERSION_1, IDS_TXPLN_CMDERROR_ENCODING_CONVERSION_1, IDS_TXPLN_CMDERROR_ENCODING_CONVERSION_1);
	gErrorTable.Add(CMDERROR_ENCODING_CONVERSION_2, IDS_TXPLN_CMDERROR_ENCODING_CONVERSION_2, IDS_TXPLN_CMDERROR_ENCODING_CONVERSION_2);
	gErrorTable.Add(CMDERROR_DELIM_TOO_LONG, IDS_TXPLN_CMDERR_DELIM_TOO_LONG, IDS_TXFMT_CMDERR_DELIM_TOO_LONG);
	gErrorTable.Add(CMDERROR_REPORT_UNEXPECTED_ARGUMENT, IDS_TXPLN_CMDERROR_REPORT_UNEXPECTED_ARGUMENT, IDS_TXFMT_CMDERROR_REPORT_UNEXPECTED_ARGUMENT);
	gErrorTable.Add(CMDERROR_NUMBER_EXPECTED_FOR_POLICY_ID, IDS_TXPLN_CMDERROR_NUMBER_EXPECTED_FOR_POLICY_ID, IDS_TXFMT_CMDERROR_NUMBER_EXPECTED_FOR_POLICY_ID);
	gErrorTable.Add(CMDERROR_POLICY_UPDATE_MISSING_PARAMS, IDS_TXPLN_CMDERROR_POLICY_UPDATE_MISSING_PARAMS, IDS_TXPLN_CMDERROR_POLICY_UPDATE_MISSING_PARAMS);
	gErrorTable.Add(CMDERROR_CONFLICT_OPTIONS_ASK_ALL, IDS_TXPLN_CMDERROR_CONFLICT_OPTIONS_ASK_ALL, IDS_TXPLN_CMDERROR_CONFLICT_OPTIONS_ASK_ALL);
	gErrorTable.Add(CMDERROR_NUMBER_EXPECTED_FOR_GROUP_ID, IDS_TXPLN_CMDERROR_NUMBER_EXPECTED_FOR_GROUP_ID, IDS_TXFMT_CMDERROR_NUMBER_EXPECTED_FOR_GROUP_ID);
	gErrorTable.Add(CMDERROR_GROUP_PARAM_NOT_APPLICABLE, IDS_TXPLN_CMDERROR_GROUP_PARAM_NOT_APPLICABLE, IDS_TXPLN_CMDERROR_GROUP_PARAM_NOT_APPLICABLE);
	gErrorTable.Add(CMDERROR_GROUP_PARAM_MISSING, IDS_TXPLN_CMDERROR_GROUP_PARAM_MISSING, IDS_TXPLN_CMDERROR_GROUP_PARAM_MISSING);
	gErrorTable.Add(CMDERROR_NUMBER_EXPECTED_FOR_REPORT_ID, IDS_TXPLN_CMDERROR_NUMBER_EXPECTED_FOR_REPORT_ID, IDS_TXFMT_CMDERROR_NUMBER_EXPECTED_FOR_REPORT_ID);
	gErrorTable.Add(CMDERROR_REPORT_TEMPLATE_UPDATE_MISSING_PARAMS, IDS_TXPLN_CMDERROR_REPORT_TEMPLATE_UPDATE_MISSING_PARAMS, IDS_TXPLN_CMDERROR_REPORT_TEMPLATE_UPDATE_MISSING_PARAMS);
	gErrorTable.Add(CMDERROR_POLICY_RULE_UPDATE_MISSING_PARAMS, IDS_TXPLN_CMDERROR_POLICY_RULE_UPDATE_MISSING_PARAMS, IDS_TXPLN_CMDERROR_POLICY_RULE_UPDATE_MISSING_PARAMS);
	gErrorTable.Add(CMDERROR_GROUP_UPDATE_MISSING_PARAMS, IDS_TXPLN_CMDERROR_GROUP_UPDATE_MISSING_PARAMS, IDS_TXPLN_CMDERROR_GROUP_UPDATE_MISSING_PARAMS);
	// <-* SECTION 2



	// *-> SECTION 3
	gErrorTable.Add(CMDERROR_INVALID_CONDITION_SYNTAX, IDS_TXPLN_CMDERROR_INVALID_CONDITION_SYNTAX, IDS_TXFMT_CMDERROR_INVALID_CONDITION_SYNTAX);
	gErrorTable.Add(CMDERROR_ASC_OR_DESC_EXPECTED_IN_ORDER, IDS_TXPLN_CMDERROR_ASC_OR_DESC_EXPECTED_IN_ORDER, IDS_TXFMT_CMDERROR_ASC_OR_DESC_EXPECTED_IN_ORDER);
	gErrorTable.Add(CMDERROR_NUMBER_EXPECTED_IN_SKIP, IDS_TXPLN_CMDERROR_NUMBER_EXPECTED_IN_SKIP, IDS_TXFMT_CMDERROR_NUMBER_EXPECTED_IN_SKIP);
	gErrorTable.Add(CMDERROR_NON_NEGATIVE_NUMBER_EXPECTED_IN_SKIP, IDS_TXPLN_CMDERROR_NON_NEGATIVE_NUMBER_EXPECTED_IN_SKIP, IDS_TXFMT_CMDERROR_NON_NEGATIVE_NUMBER_EXPECTED_IN_SKIP);
	gErrorTable.Add(CMDERROR_NUMBER_EXPECTED_IN_LIMIT, IDS_TXPLN_CMDERROR_NUMBER_EXPECTED_IN_LIMIT, IDS_TXFMT_CMDERROR_NUMBER_EXPECTED_IN_LIMIT);
	gErrorTable.Add(CMDERROR_NON_NEGATIVE_NUMBER_EXPECTED_IN_LIMIT, IDS_TXPLN_CMDERROR_NON_NEGATIVE_NUMBER_EXPECTED_IN_LIMIT, IDS_TXFMT_CMDERROR_NON_NEGATIVE_NUMBER_EXPECTED_IN_LIMIT);
	gErrorTable.Add(CMDERROR_FLAG_EXPECTED, IDS_TXPLN_CMDERROR_FLAG_EXPECTED, IDS_TXFMT_CMDERROR_FLAG_EXPECTED);
	gErrorTable.Add(CMDERROR_UNSUPPORTED_FLAG, IDS_TXPLN_CMDERROR_UNSUPPORTED_FLAG, IDS_TXFMT_CMDERROR_UNSUPPORTED_FLAG);
	gErrorTable.Add(CMDERROR_INVALID_BOOL_ARGUMENT, IDS_TXPLN_CMDERROR_INVALID_BOOL_ARGUMENT, IDS_TXFMT_CMDERROR_INVALID_BOOL_ARGUMENT);
	gErrorTable.Add(CMDERROR_INVALID_FORMATTING, IDS_TXPLN_CMDERROR_INVALID_FORMATTING, IDS_TXFMT_CMDERROR_INVALID_FORMATTING);
	gErrorTable.Add(CMDERROR_INVALID_ENCODING, IDS_TXPLN_CMDERROR_INVALID_ENCODING, IDS_TXFMT_CMDERROR_INVALID_ENCODING);
	gErrorTable.Add(CMDERROR_INVALID_MODE, IDS_TXPLN_CMDERROR_INVALID_MODE, IDS_TXFMT_CMDERROR_INVALID_MODE);
	gErrorTable.Add(CMDERROR_UNABLE_TO_READ_FILE, IDS_TXPLN_CMDERROR_UNABLE_TO_READ_FILE, IDS_TXFMT_CMDERROR_UNABLE_TO_READ_FILE);
	gErrorTable.Add(CMDERROR_ENCODING_CONVERSION_FAILED, IDS_TXPLN_CMDERROR_ENCODING_CONVERSION_FAILED, IDS_TXFMT_CMDERROR_ENCODING_CONVERSION_FAILED);
	gErrorTable.Add(CMDERROR_TOO_MANY_LEVELS_IN_FILE_REPLACING, IDS_TXPLN_CMDERROR_TOO_MANY_LEVELS_IN_FILE_REPLACING, IDS_TXPLN_CMDERROR_TOO_MANY_LEVELS_IN_FILE_REPLACING);
	gErrorTable.Add(CMDERROR_FILE_REPLACING_LOOP_DETECTED, IDS_TXPLN_CMDERROR_FILE_REPLACING_LOOP_DETECTED, IDS_TXPLN_CMDERROR_FILE_REPLACING_LOOP_DETECTED);
	gErrorTable.Add(CMDERROR_FILE_REPLACING_MISSING_END, IDS_TXPLN_CMDERROR_FILE_REPLACING_MISSING_END, IDS_TXPLN_CMDERROR_FILE_REPLACING_MISSING_END);
	gErrorTable.Add(CMDERROR_INVALID_HEADER_TYPE, IDS_TXPLN_CMDERROR_INVALID_HEADER_TYPE, IDS_TXFMT_CMDERROR_INVALID_HEADER_TYPE);
	gErrorTable.Add(CMDERROR_NUMBER_EXPECTED_IN_ID_LIST, IDS_TXPLN_CMDERROR_NUMBER_EXPECTED_IN_ID_LIST, IDS_TXFMT_CMDERROR_NUMBER_EXPECTED_IN_ID_LIST);
	gErrorTable.Add(CMDERROR_INVALID_DATETIME, IDS_TXPLN_CMDERROR_INVALID_DATETIME, IDS_TXFMT_CMDERROR_INVALID_DATETIME);
	gErrorTable.Add(CMDERROR_INVALID_BOOL, IDS_TXPLN_CMDERROR_INVALID_BOOL, IDS_TXFMT_CMDERROR_INVALID_BOOL);
	gErrorTable.Add(CMDERROR_INVALID_ITEM_ONE_OF_EXPECTED, IDS_TXPLN_CMDERROR_INVALID_ITEM_ONE_OF_EXPECTED, IDS_TXFMT_CMDERROR_INVALID_ITEM_ONE_OF_EXPECTED);
	gErrorTable.Add(CMDERROR_POSITIVE_NUMBER_EXPECTED, IDS_TXPLN_CMDERROR_POSITIVE_NUMBER_EXPECTED, IDS_TXFMT_CMDERROR_POSITIVE_NUMBER_EXPECTED);
	gErrorTable.Add(CMDERROR_INVALID_FIELD_FORMATTING, IDS_TXPLN_CMDERROR_INVALID_FIELD_FORMATTING, IDS_TXFMT_CMDERROR_INVALID_FIELD_FORMATTING);
	gErrorTable.Add(CMDERROR_LEFT_PARENTHESIS_ONLY_AFTER_IN, IDS_TXPLN_CMDERROR_LEFT_PARENTHESIS_ONLY_AFTER_IN, IDS_TXPLN_CMDERROR_LEFT_PARENTHESIS_ONLY_AFTER_IN);
	gErrorTable.Add(CMDERROR_MISSING_RIGHT_PARENTHESIS, IDS_TXPLN_CMDERROR_MISSING_RIGHT_PARENTHESIS, IDS_TXPLN_CMDERROR_MISSING_RIGHT_PARENTHESIS);
	gErrorTable.Add(CMDERROR_MISSING_COMMA, IDS_TXPLN_CMDERROR_MISSING_COMMA, IDS_TXPLN_CMDERROR_MISSING_COMMA);
	gErrorTable.Add(CMDERROR_VALUE_EXPECTED, IDS_TXPLN_CMDERROR_VALUE_EXPECTED, IDS_TXPLN_CMDERROR_VALUE_EXPECTED);
	gErrorTable.Add(CMDERROR_SOME_ERRORS_IGNORED, IDS_TXPLN_CMDERROR_SOME_ERRORS_IGNORED, IDS_TXPLN_CMDERROR_SOME_ERRORS_IGNORED);
	gErrorTable.Add(CMDERROR_MISSING_PATH_ARGUMENT, IDS_TXPLN_CMDERROR_MISSING_PATH_ARGUMENT, IDS_TXPLN_CMDERROR_MISSING_PATH_ARGUMENT);
	gErrorTable.Add(CMDERROR_UNKNOWN_CURRENT_SCRIPT_PATH, IDS_TXPLN_CMDERROR_UNKNOWN_CURRENT_SCRIPT_PATH, IDS_TXPLN_CMDERROR_UNKNOWN_CURRENT_SCRIPT_PATH);
	gErrorTable.Add(CMDERROR_UNABLE_TO_SET_CURRENT_WORKING_DIRECTORY, IDS_TXPLN_CMDERROR_UNABLE_TO_SET_CURRENT_WORKING_DIRECTORY, IDS_TXFMT_CMDERROR_UNABLE_TO_SET_CURRENT_WORKING_DIRECTORY);
	gErrorTable.Add(CMDERROR_NO_USERNAME_FOR_DOMAIN, IDS_TXPLN_CMDERROR_NO_USERNAME_FOR_DOMAIN, IDS_TXPLN_CMDERROR_NO_USERNAME_FOR_DOMAIN);
	gErrorTable.Add(CMDERROR_USERNAME_CANNOT_BE_EMPTY, IDS_TXPLN_CMDERROR_USERNAME_CANNOT_BE_EMPTY, IDS_TXPLN_CMDERROR_USERNAME_CANNOT_BE_EMPTY);
	gErrorTable.Add(CMDERROR_CANNOT_WRITE_TO_FILE, IDS_TXPLN_CMDERROR_CANNOT_WRITE_TO_FILE, IDS_TXFMT_CMDERROR_CANNOT_WRITE_TO_FILE);
	gErrorTable.Add(CMDERROR_MISSING_END_QUOTE, IDS_TXPLN_CMDERROR_MISSING_END_QUOTE, IDS_TXPLN_CMDERROR_MISSING_END_QUOTE);
	gErrorTable.Add(CMDERROR_PATH_IS_A_DIRECTORY, IDS_TXPLN_CMDERROR_PATH_IS_A_DIRECTORY, IDS_TXFMT_CMDERROR_PATH_IS_A_DIRECTORY);
	gErrorTable.Add(CMDERROR_FILE_NOT_WRITABLE, IDS_TXPLN_CMDERROR_FILE_NOT_WRITABLE, IDS_TXFMT_CMDERROR_FILE_NOT_WRITABLE);
	gErrorTable.Add(CMDERROR_FILE_CANNOT_BE_CREATED, IDS_TXPLN_CMDERROR_FILE_CANNOT_BE_CREATED, IDS_TXFMT_CMDERROR_FILE_CANNOT_BE_CREATED);
	gErrorTable.Add(CMDERROR_NUMBER_EXPECTED_FOR_POLICY_RULE_ID, IDS_TXPLN_CMDERROR_NUMBER_EXPECTED_FOR_POLICY_RULE_ID, IDS_TXFMT_CMDERROR_NUMBER_EXPECTED_FOR_POLICY_RULE_ID);
	gErrorTable.Add(CMDERROR_NUMBER_EXPECTED_FOR_TEMPLATE_ID, IDS_TXPLN_CMDERROR_NUMBER_EXPECTED_FOR_TEMPLATE_ID, IDS_TXFMT_CMDERROR_NUMBER_EXPECTED_FOR_TEMPLATE_ID);
	// <-* SECTION 3



	// *-> SECTION 4
		ADD_ERROR(CMDERROR_SCRIPT_TOO_NESTED);
		ADD_ERROR(CMDERROR_SCRIPT_INFINITE_LOOP);
		ADD_ERROR(CMDERROR_HELP_UNKNOWN_COMMAND);

		ADD_ERROR(CMDERROR_COMMAND_UNEXPECTED_ARGUMENT);
		ADD_ERROR(CMDERROR_COMMAND_UNEXPECTED_ARGUMENT2);
		ADD_ERROR(CMDERROR_COMMAND_TOO_MANY_ARGUMENTS);
		ADD_ERROR(CMDERROR_COMMAND_MISSING_ARGUMENT);
		ADD_ERROR(CMDERROR_UNKNOWN_DATA_TYPE_REQUESTED);

		ADD_ERROR(CMDERROR_LIST_NOT_SUPPORTED);
		ADD_ERROR(CMDERROR_INVALID_ARGUMENT_VALUE);
		ADD_ERROR(CMDERROR_PASSWORD_NOT_MATCHING);
	// <-* SECTION 4
};

