/**
 * @file Autocomplete.h
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#pragma once

#include "../ScriptInterpreter.h"


// this object is sent to scriptinterpreter when asking for autocomplete there
class AutocompleteCollector
{
public:
	AutocompleteCollector(const std::wstring &openWord, std::vector<std::wstring> &completions) : openWord(openWord), completions(completions) {}
	void Add(const std::wstring &completion) 
	{
		if (!completion.empty() && std::find(completions.begin(), completions.end(), completion)==completions.end()) 
			completions.push_back(completion); 
	}
	std::wstring openWord;
protected:
	std::vector<std::wstring> &completions;
};



class Autocomplete
{
public:
	Autocomplete(ScriptInterpreter &scriptInterpreter);
	bool IsReset();
	void Reset();
	int GetNext(const std::wstring &commandLine, std::wstring &completion); // returns number of letters from the command which are untouched, the rest should be replaced by suggested completion

private:
	void BuildCommandAutocomplete(const std::wstring &lineLocked, const std::wstring &lineOpened);
	void BuildFlagsAutocomplete(const std::wstring &lineOpened);


	void Build(const std::wstring &commandLine);

	ScriptInterpreter &scriptInterpreter;
	int inputLength;
	int lastIndex;
	std::vector<std::wstring> completions;//could be std::set, but we might possible want to have it custom-ordered
};

