/**
 * @file CCommand_Encrypt.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */


#include "set_nod.h"
#include <Common/era_api/era_api.h>
#include <Common/scope_exit.h>
#include <Common/era_api/err_codes.h>
#include "../util/apiUtils.h"
#include "../util/stringUtils.h"
#include "../EracmdExceptions.h"
#include "../ConsoleKeyboardInput.h"
#include "Command_Encrypt.h"

Command_Encrypt::Command_Encrypt(ScriptInterpreter *scriptInterpreter) : scriptInterpreter(scriptInterpreter)
{

}

void Command_Encrypt::Execute(elementArray& elements, OutputHandler& out)
{
	//type
	Era_PasswordEncryptionType passType;
	std::wstring typeStr = GetArgumentAsString(0);
	if (_wcsicmp(typeStr.c_str(), L"server")==0)
		passType = ERA_PASSWORD_ENCRYPTIONTYPE__SERVER;
	else if (_wcsicmp(typeStr.c_str(), L"other")==0)
		passType = ERA_PASSWORD_ENCRYPTIONTYPE__OTHER;
	else if (_wcsicmp(typeStr.c_str(), L"hash")==0)
		passType = ERA_PASSWORD_ENCRYPTIONTYPE__HASH;
	else if (_wcsicmp(typeStr.c_str(), L"int64")==0)
		passType = ERA_PASSWORD_ENCRYPTIONTYPE__NUM;

	//passwords
	std::wstring password=GetArgumentAsString(1);
	std::shared_ptr<Element> passElement = GetArgument(1);
	if (passElement==NULL)
	{
		ConsoleKeyboardInput keyInput(*scriptInterpreter, true);
		std::wstring passwordMessage = LoadStringFromResource(IDS_ENTER_PASSWORD);
		std::wcout << passwordMessage << L" ";
		keyInput.GetInput();
		password = keyInput.GetLine();
		std::wcout << std::endl;
	}
	else
	{
		password=GetArgumentAsString(1);
	}

	//get required buffer length
	Era_EncryptedPassword* encryptedPassword=NULL;
	uint32_t result = era_EncryptPassword(passType, password.c_str(), &encryptedPassword);
	if (result != 0)
		throw EraCMDException_FromApi(result);

	scope_exit se_serverInfo(std::bind(era_EncryptedPassword_delete, encryptedPassword));

	std::wstring resultStr = era_EncryptedPassword_GetEncryptedPassword(encryptedPassword);
	resultStr+=L"\n";
	out.PrintRaw(resultStr.c_str());
}

