/**
 * @file Command_ErrMsg.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_ErrMsg.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/era_api/err_lookup.h>
#include "../util/stringUtils.h"
#include "../EracmdExceptions.h"
#include "../Element.h"

void Command_ErrMsg::Execute(elementArray &elements, OutputHandler &out)
{
	std::wstring arg(GetArgument(0)->GetString());

	DWORD code = 0;
	try
	{
		code = WstringToInt(arg);
	} catch (...)
	{
		throw EraCMDException(CMDERROR_INVALID_PARAMETER, arg.c_str());
	}

	std::wstring text;

	if ((code >= CMDERROR_BASE) || (code == CMDERROR_SOME_ERRORS_IGNORED))
	{
		// command console errors
		text = GetErrorMessage(code);
	}
	else
	{
		// ERA API errors
		
		wchar_t *msg;
		uint32_t rc = era_GetErrorMessage2(code, &msg);
		if (rc != ERAERROR_SUCCESS)
			throw EraCMDException_FromApi(rc);

		text = msg;
		era_DeleteString(msg);
	}
	std::wstringstream ss;
	ss << L"Error code " << arg << L":" << std::endl << text << std::endl;
	out.PrintRaw(ss.str());
}

