/**
 * @file Command_History.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_History.h"

#include <Common/era_api/err_codes.h>
#include "../Util/fileUtils.h"
#include "../Util/stringUtils.h"
#include "../ConsoleHistory.h"
#include "../Element.h"
#include "../EracmdExceptions.h"


void Command_History::Execute(elementArray &elements, OutputHandler &out)
{
	std::wstring action = GetArgumentAsString(0);
	if (action.empty()) // empty?
	{
		std::wstring tx = LoadStringFromResource(gCmdHistory.Active ? IDS_HELP_HISTORY_IS_TRUE : IDS_HELP_HISTORY_IS_FALSE);
		std::wstringstream ss;
		ss << tx;
		ss << std::endl;
		out.PrintRaw(ss.str());
	}
	else
	if (_wcsicmp(action.c_str(), L"TRUE") == 0)
	{
		gCmdHistory.Active = true;
	}
	else
	if (_wcsicmp(action.c_str(), L"FALSE") == 0)
	{
		gCmdHistory.Active = false;
	}
	else
	if (_wcsicmp(action.c_str(), L"CLEAR") == 0)
	{
		gCmdHistory.Clear();
		LPCWSTR file = gCmdHistory.FileName.c_str();
		if (FileExists(file))
		{
			HANDLE h = CreateFile(file, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL);
			if (h != INVALID_HANDLE_VALUE)
			{
				CloseHandle(h);
			}
		}
	}
	else
	if (_wcsicmp(action.c_str(), L"LIST") == 0)
	{
		std::wstringstream ss;
		gCmdHistory.GoBegin();
		size_t cnt = gCmdHistory.GetCount();
		for (size_t i = 0; i < cnt; ++i)
		{
			std::wstring cmd = gCmdHistory.GetNextItem();
			ss << cmd;
			if (i < cnt - 1)
				ss << std::endl;
		}
		out.PrintRaw(ss.str());
	}
	else
	{
		throw EraCMDException(CMDERROR_INVALID_PARAMETER, action.c_str());
	}
}

