/**
 * @file Command_LicenseDetails.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_LicenseDetails.h"

#include <Common/scope_exit.h>
#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include "../util/stringUtils.h"
#include "../EracmdExceptions.h"

void Command_LicenseDetails::Execute(elementArray &elements, OutputHandler &out)
{
	Era_LicenseInfo *licenseInfo;

	uint32_t rc = era_GetLicenseInfo(&licenseInfo);
	if (rc != ERAERROR_SUCCESS)
	{
		throw EraCMDException_FromApi(rc);
	}
	scope_exit se_LicenseInfo(std::bind(era_LicenseInfo_delete, licenseInfo));

	#define LICENSE_DETAILS_COLUMNS 7

	StringVector header(LICENSE_DETAILS_COLUMNS);
	header[0] = L"licensed_to";
	header[1] = L"license_clients";
	header[2] = L"license_expires";
	header[3] = L"license_product_type";
	header[4] = L"license_id";
	header[5] = L"license_productid";
	header[6] = L"license_partnerid";

	StringVector prettyHeader(LICENSE_DETAILS_COLUMNS);
	prettyHeader[0] = LoadStringFromResource(IDS_LICENSE_LICENSED_TO);
	prettyHeader[1] = LoadStringFromResource(IDS_LICENSE_CLIENTS);
	prettyHeader[2] = LoadStringFromResource(IDS_LICENSE_EXPIRES);
	prettyHeader[3] = LoadStringFromResource(IDS_LICENSE_PRODUCT_TYPE);
	prettyHeader[4] = LoadStringFromResource(IDS_LICENSE_ID);
	prettyHeader[5] = LoadStringFromResource(IDS_LICENSE_PRODUCT_ID);
	prettyHeader[6] = LoadStringFromResource(IDS_LICENSE_PARTNER_ID);

	std::vector<StringVector> rows;

	uint32_t licensesCount = era_LicenseInfo_GetLicensesCount(licenseInfo);
	for (DWORD i = 0; i < licensesCount; ++i)
	{
		const Era_License *lic = era_LicenseInfo_GetLicense(licenseInfo, i);

		int64_t time = era_License_GetExpirationTime(lic);
		struct tm expt;
		_gmtime64_s(&expt, &time);
		WCHAR expirationTime[100];
		swprintf_s(expirationTime, _countof(expirationTime), L"%04d-%02d-%02d", expt.tm_year + 1900, expt.tm_mon + 1, expt.tm_mday);

		StringVector row(LICENSE_DETAILS_COLUMNS);
		row[0] = era_License_GetCustomerName(lic);
		row[1] = std::to_wstring((long long)era_License_GetUserCount(lic));
		row[2] = expirationTime;
		row[3] = era_License_GetProductName(lic);
		row[4] = std::to_wstring((long long)era_License_GetId(lic));
		row[5] = std::to_wstring((long long)era_License_GetProductId(lic));
		row[6] = era_License_GetDistGuid(lic);
		rows.push_back(row);
	}

	TableReader_Const reader(header, prettyHeader, rows);
	out.Print(reader);
}

