/**
 * @file Command_Path.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_Path.h"

#include <Common/era_api/err_codes.h>
#include "../Element.h"
#include "../EracmdExceptions.h"

Command_Path::Command_Path(const std::wstring &currentScriptPath)
:
	currentScriptPath(currentScriptPath)
{
}

void Command_Path::Execute(elementArray &elements, OutputHandler &out)
{
	std::shared_ptr<Element> actionElement = GetArgument(0);
	if (actionElement && (::_wcsicmp(actionElement->GetString().c_str(), L"get") != 0))
	{
		std::wstring pathToSet;

		if (::_wcsicmp(actionElement->GetString().c_str(), L"set") == 0)
		{
			std::shared_ptr<Element> pathElement = GetArgument(1);
			if (! pathElement)
			{
				throw EraCMDException(CMDERROR_MISSING_PATH_ARGUMENT);
			}

			std::wstring path = pathElement->GetString();
			wchar_t lastChar = path[path.size() - 1];
			if ((lastChar != L'\\') && (lastChar != L'/'))
			{
				path.push_back(L'\\');
			}

			pathToSet = path;
		}

		if (::_wcsicmp(actionElement->GetString().c_str(), L"script") == 0)
		{
			if (GetArgument(1))
			{
				throw EraCMDException(CMDERROR_COMMAND_TOO_MANY_ARGUMENTS, GetArgument(1)->GetString().c_str());
			}

			if (currentScriptPath.empty())
			{
				throw EraCMDException(CMDERROR_UNKNOWN_CURRENT_SCRIPT_PATH);
			}

			wchar_t drive[_MAX_DRIVE];
			wchar_t dir[_MAX_DIR];
			if (_wsplitpath_s(currentScriptPath.c_str(), drive, _MAX_DRIVE, dir, _MAX_DIR, NULL, 0, NULL, 0) != 0)
			{
				throw EraCMDException(CMDERROR_UNKNOWN_CURRENT_SCRIPT_PATH);
			}

			pathToSet = std::wstring(drive) + std::wstring(dir);
		}

		if (_wchdir(pathToSet.c_str()) != 0)
		{
			throw EraCMDException(CMDERROR_UNABLE_TO_SET_CURRENT_WORKING_DIRECTORY, pathToSet.c_str());
		}
	}
	else
	{
		if (GetArgument(1))
		{
			throw EraCMDException(CMDERROR_COMMAND_TOO_MANY_ARGUMENTS, GetArgument(1)->GetString().c_str());
		}

		wchar_t *buffer;

		if ((buffer = _wgetcwd(NULL, 0)) != NULL)
		{
			out.PrintRaw(buffer);
			free(buffer);
		}
	}
}

