/**
 * @file Command_PolicyBase.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_PolicyBase.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"

int Command_PolicyBase::GetNumParam(unsigned int argNum)
{
	std::shared_ptr<Element> element = GetArgument(argNum);
	uint32_t num = 0;
	try
	{
		num = WstringToUint(element->GetString());
	}
	catch (...)
	{
		throw EraCMDException(CMDERROR_NUMBER_EXPECTED_FOR_POLICY_ID, element->GetString().c_str());
	}

	return num;
}

bool Command_PolicyBase::FillPolicyParams(const int argNumOffset, bool parentIdSecond, Era_PolicyParams &params)
{

	bool bAtLeastOneParam = false;

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 0))
	{
		era_PolicyParams_SetName(&params, element->GetString().c_str());
		bAtLeastOneParam = true;
	}

	if (std::shared_ptr<Element> element = GetArgument(parentIdSecond ? argNumOffset + 1 : argNumOffset + 2))
	{

		const std::wstring &policyIdArg = element->GetString();
		if (_wcsicmp(policyIdArg.c_str(), L"!nopolicy") == 0)
		{
			era_PolicyParams_SetNoParent(&params);
		}
		else
		{
			uint32_t num = 0;
			try
			{
				num = WstringToUint(policyIdArg);
			}
			catch (...)
			{
				throw EraCMDException(CMDERROR_NUMBER_EXPECTED_FOR_POLICY_ID, element->GetString().c_str());
			}
			era_PolicyParams_SetParentId(&params, num);
		}
		bAtLeastOneParam = true;
	}

	if (std::shared_ptr<Element> element = GetArgument(parentIdSecond ? argNumOffset + 2 : argNumOffset + 1))
	{
		era_PolicyParams_SetConfigXml(&params, element->GetString().c_str());
		bAtLeastOneParam = true;
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 3))
	{
		era_PolicyParams_SetDescription(&params, element->GetString().c_str());
		bAtLeastOneParam = true;
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 4))
	{
		if (::ToBool(element->GetString()))
			era_PolicyParams_OverrideAnyChild(&params, ERA_POLICY_FLAG_STATE__SET);
		else
			era_PolicyParams_OverrideAnyChild(&params, ERA_POLICY_FLAG_STATE__UNSET);
		bAtLeastOneParam = true;
	}
	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 5))
	{
		if (::ToBool(element->GetString()))
			era_PolicyParams_DownReplicable(&params, ERA_POLICY_FLAG_STATE__SET);
		else
			era_PolicyParams_DownReplicable(&params, ERA_POLICY_FLAG_STATE__UNSET);
		bAtLeastOneParam = true;
	}
	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 6))
	{
		if (::ToBool(element->GetString()))
			era_PolicyParams_SetSetAsDefaultForClients(&params, ERA_POLICY_FLAG_STATE__SET);
		else
			era_PolicyParams_SetSetAsDefaultForClients(&params, ERA_POLICY_FLAG_STATE__UNSET);
		bAtLeastOneParam = true;
	}
	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 7))
	{
		if (::ToBool(element->GetString()))
			era_PolicyParams_SetSetAsDefaultForLowerServers(&params, ERA_POLICY_FLAG_STATE__SET);
		else
			era_PolicyParams_SetSetAsDefaultForLowerServers(&params, ERA_POLICY_FLAG_STATE__UNSET);
		bAtLeastOneParam = true;
	}

	return bAtLeastOneParam;
}

void Command_PolicyBase::FillReplacement(const int argNum, PolicyReplacementPtr &replacement)
{
	if (std::shared_ptr<Element> element = GetArgument(argNum))
	{
		replacement.reset(era_PolicyReplacement_new());

		const std::wstring elementStr = element->GetString();

		if (::_wcsicmp(elementStr.c_str(), L"!DefaultClientsPolicy") == 0)
		{
			era_PolicyReplacement_SelectDefaultClientsPolicy(replacement.get());
		}
		else if (::_wcsicmp(elementStr.c_str(), L"!DefaultUpperServerPolicy") == 0)
		{
			era_PolicyReplacement_SelectDefaultUpperServerPolicy(replacement.get());
		}
		else if (::_wcsicmp(elementStr.c_str(), L"!NotAvailable") == 0)
		{
			era_PolicyReplacement_SelectNotAvailable(replacement.get());
		}
		else
		{
			uint32_t id = 0;
			try
			{
				id = ::WstringToUint(elementStr);
			}
			catch (...)
			{
				throw EraCMDException(CMDERROR_NUMBER_EXPECTED_FOR_POLICY_ID, elementStr.c_str());
			}

			era_PolicyReplacement_SelectPolicy(replacement.get(), id);
		}
	}
}
