/**
 * @file Command_PolicyRuleCreate.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_PolicyRuleCreate.h"

#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../EracmdExceptions.h"

void Command_PolicyRuleCreate::Execute(elementArray &elements, OutputHandler &out)
{
	uint32_t newRuleId = -1;

	std::wstring xmlFile = GetArgumentAsString(0);
	std::wstring name = GetArgumentAsString(1);
	int policyId = 0;
	std::wstring policyIdStr = GetArgumentAsString(2);
	std::wstring desc = GetArgument(3) ? GetArgumentAsString(3) : L"";
	Era_PolicyRulePriority priority = _wcsicmp(GetArgumentAsString(4).c_str(), L"top") == 0 ? ERA_POLICY_RULE_PRIORITY_TOP : ERA_POLICY_RULE_PRIORITY_BOTTOM;
	int enabled = _wcsicmp(GetArgumentAsString(5).c_str(), L"false") == 0 ? 0 : 1;


//policy ID
	if (_wcsicmp(policyIdStr.c_str(), L"!defaultclientspolicy") == 0)
		policyId = 0;
	else
	{
		try
		{
			policyId = WstringToUint(policyIdStr);
		}
		catch (...)
		{
			throw EraCMDException(CMDERROR_NUMBER_EXPECTED_FOR_POLICY_ID, policyIdStr.c_str());
		}
	}

	Era_PolicyRuleParams* params = era_PolicyRuleParams_new();
	scope_exit se_params(std::bind(era_PolicyRuleParams_delete, params));
	
	era_PolicyRuleParams_SelectPolicyId(params, policyId);
	era_PolicyRuleParams_SetEnableState(params, enabled);
	era_PolicyRuleParams_SetPriority(params, priority);
	era_PolicyRuleParams_SetDescription(params, desc.c_str());
	era_PolicyRuleParams_SetName(params, name.c_str());
	era_PolicyRuleParams_SetRuleXml(params, xmlFile.c_str());

	uint32_t rc = era_CreatePolicyRule(&newRuleId, params);
	if (rc != ERAERROR_SUCCESS)
	{
		throw EraCMDException_FromApi(rc);
	}
	

	StringVector header(1);
	header[0] = L"rule_id";
	StringVector prettyHeader(1);
	prettyHeader[0] = LoadStringFromResource(IDS_RULE_ID);
	std::vector<StringVector> rows;
	StringVector data(1);
	data[0] = std::to_wstring((long long)newRuleId);
	rows.push_back(data);
	TableReader_Const reader(header, prettyHeader, rows);
	out.Print(reader);
}

