/**
 * @file Command_PolicyRuleUpdate.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_PolicyRuleUpdate.h"

#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"

void Command_PolicyRuleUpdate::Execute(elementArray &elements, OutputHandler &out)
{
	uint32_t updatedPolicyRuleId = 0;
	{
		const std::wstring elementStr = GetArgument(0)->GetString();

		try
		{
			updatedPolicyRuleId = ::WstringToUint(elementStr);
		}
		catch (...)
		{
			throw EraCMDException(CMDERROR_NUMBER_EXPECTED_FOR_POLICY_RULE_ID, elementStr.c_str());
		}
	}

	Era_PolicyRuleParams* params = era_PolicyRuleParams_new();
	scope_exit se_params(std::bind(era_PolicyRuleParams_delete, params));

	bool bAtLeastOneParam = false;

	if (std::shared_ptr<Element> element = GetArgument(1))
	{
		era_PolicyRuleParams_SetRuleXml(params, element->GetString().c_str());
		bAtLeastOneParam = true;
	}

	if (std::shared_ptr<Element> element = GetArgument(2))
	{
		era_PolicyRuleParams_SetDescription(params, element->GetString().c_str());
		bAtLeastOneParam = true;
	}

	if (std::shared_ptr<Element> element = GetArgument(3))
	{
		std::wstring policyIdStr = element->GetString();

		uint32_t policyId;

		if (_wcsicmp(policyIdStr.c_str(), L"!defaultclientspolicy") == 0)
			policyId = 0;
		else
		{
			try
			{
				policyId = WstringToUint(policyIdStr);
			}
			catch (...)
			{
				throw EraCMDException(CMDERROR_NUMBER_EXPECTED_FOR_POLICY_ID, policyIdStr.c_str());
			}
		}

		era_PolicyRuleParams_SelectPolicyId(params, policyId);
		bAtLeastOneParam = true;
	}

	if (std::shared_ptr<Element> element = GetArgument(4))
	{
		std::wstring priorityStr = element->GetString();

		if (_wcsicmp(priorityStr.c_str(), L"up") == 0)
			era_PolicyRuleParams_SetPriority(params, ERA_POLICY_RULE_PRIORITY_UP);
		if (_wcsicmp(priorityStr.c_str(), L"down") == 0)
			era_PolicyRuleParams_SetPriority(params, ERA_POLICY_RULE_PRIORITY_DOWN);
		if (_wcsicmp(priorityStr.c_str(), L"top") == 0)
			era_PolicyRuleParams_SetPriority(params, ERA_POLICY_RULE_PRIORITY_TOP);
		if (_wcsicmp(priorityStr.c_str(), L"bottom") == 0)
			era_PolicyRuleParams_SetPriority(params, ERA_POLICY_RULE_PRIORITY_BOTTOM);

		bAtLeastOneParam = true;
	}

	if (std::shared_ptr<Element> element = GetArgument(5))
	{
		std::wstring enabledStr = element->GetString();

		if (_wcsicmp(enabledStr.c_str(), L"true") == 0)
			era_PolicyRuleParams_SetEnableState(params, 1);
		if (_wcsicmp(enabledStr.c_str(), L"false") == 0)
			era_PolicyRuleParams_SetEnableState(params, 0);

		bAtLeastOneParam = true;
	}

	if (!bAtLeastOneParam)
		throw EraCMDException(CMDERROR_POLICY_RULE_UPDATE_MISSING_PARAMS);

	uint32_t result_policyRuleUpdate = era_PolicyRule_update(updatedPolicyRuleId, params);
	if (result_policyRuleUpdate != 0)
	{
		throw EraCMDException_FromApi(result_policyRuleUpdate);
	}
}

