/**
 * @file Command_ScanLog.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_ScanLog.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../EracmdExceptions.h"
#include "../Element.h"

Command_ScanLog::Command_ScanLog(CommandFlagsStack &commandFlagsStack)
	: commandFlagsStack(commandFlagsStack)
{
}

void Command_ScanLog::Execute(elementArray &elements, OutputHandler &out)
{
	std::wstring idStr = GetArgument(0)->GetString();
	uint64_t id = 0;
	try
	{
		id = WstringToUInt64(idStr);
	} catch (...)
	{
		throw EraCMDException(CMDERROR_INVALID_PARAMETER, idStr.c_str());
	}

	Era_ScanLog *scanlog;

	uint32_t rc = era_OpenScanLog(id, &scanlog);
	if (rc != ERAERROR_SUCCESS)
	{
		throw EraCMDException_FromApi(rc);
	}

	scope_exit se_LicenseInfo(std::bind(era_CloseScanLog, scanlog));

	bool prettyFormat = commandFlagsStack.CurrentFlags().fieldFormatting == CommandFlags::FIELD_PRETTY;

	StringVector header(2);
	header[0] = L"level";
	header[1] = L"log";

	StringVector prettyHeader(2);
	prettyHeader[0] = LoadStringFromResource(IDS_LEVEL);
	prettyHeader[1] = LoadStringFromResource(IDS_LOG);

	std::vector<StringVector> rows;

	const wchar_t *level;
	const wchar_t *line;

	StringVector row(2);

	for(;;)
	{
		rc = era_GetScanLogRow(scanlog, prettyFormat, &level, &line);
		if (rc == APIERROR_NO_MORE_DATA)
			break;
		if (rc != ERAERROR_SUCCESS)
		{
			throw EraCMDException_FromApi(rc);
		}

		row[0] = level;
		row[1] = line;
		rows.push_back(row);
	}

	TableReader_Const reader(header, prettyHeader, rows);
	out.Print(reader);
}

