/**
 * @file Command_ShowColumns.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_ShowColumns.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"

void Command_ShowColumns::Execute(elementArray &elements, OutputHandler &out)
{
	Era_TableReader *columnsInfo;

	std::wstring tableName = GetArgument(0)->GetString();
	uint32_t result_getColumnsInfo = era_GetColumnsInfo(tableName.c_str(), &columnsInfo);
	if (result_getColumnsInfo != 0)
	{
		throw EraCMDException_FromApi(result_getColumnsInfo);
	}
	scope_exit se_columnsInfo(std::bind(era_TableReader_delete, columnsInfo));

	StringVector header(4);
	header[0] = L"keyword";
	header[1] = L"name";
	header[2] = L"type";
	header[3] = L"values";

	StringVector prettyHeader(4);
	prettyHeader[0] = LoadStringFromResource(IDS_SHOWCOLUMNS_COLNAME_KEYWORD);
	prettyHeader[1] = LoadStringFromResource(IDS_SHOWCOLUMNS_COLNAME_NAME);
	prettyHeader[2] = LoadStringFromResource(IDS_SHOWCOLUMNS_COLNAME_TYPE);
	prettyHeader[3] = LoadStringFromResource(IDS_SHOWCOLUMNS_COLNAME_VALUES);

	uint32_t columnsCount = era_TableReader_GetColumnsCount(columnsInfo);
	std::vector<StringVector> rows(columnsCount);

	for (uint32_t i = 0; i < columnsCount; ++i)
	{
		const Era_ColumnInfo *columnInfo = era_TableReader_GetColumn(columnsInfo, i);
		StringVector row(4);
		row[0] = era_ColumnInfo_GetHeader(columnInfo);
		row[1] = era_ColumnInfo_GetPrettyHeader(columnInfo);
		row[2] = era_ColumnInfo_GetTypeName(columnInfo);

		{
			uint32_t typeValuesCount = era_ColumnInfo_GetTypeValuesCount(columnInfo);
			std::wostringstream ss;
			for (uint32_t j = 0; j < typeValuesCount; ++j)
			{
				if (j > 0)
				{
					ss << L", ";
				}
				ss << era_ColumnInfo_GetTypeValue(columnInfo, j);
			}
			row[3] = ss.str();
		}

		rows[i] = std::move(row);
	}

	TableReader_Const reader(header, prettyHeader, rows);
	out.Print(reader);
}

