/**
 * @file Command_TaskUpdate.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_TaskUpdate.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/apiUtils.h"
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"

void Command_TaskUpdate::Execute(elementArray &elements, OutputHandler &out)
{
	Era_TaskParams *params = era_TaskParams_new();
	scope_exit se_query(std::bind(era_TaskParams_delete, params));

	FillTaskParams(0, *params);
	Era_ClientList *clients = ElementToClientList(GetArgument(0));
	scope_exit se_clients(std::bind(era_ClientList_delete, clients));

	bool windows = true;
	bool linux3 = true;
	bool linux = true;
	bool mobile = true;
	FillExcludeBools(5, windows, linux3, linux, mobile);

	std::wstring windowsProfileName;
	const wchar_t *windowsProfileName_orNull = NULL;
	if (std::shared_ptr<Element> element = GetArgument(6))
	{
		windowsProfileName = element->GetString();
		windowsProfileName_orNull = windowsProfileName.c_str();
	}

	unsigned int randomDelay_maxSeconds = 0;
	if (std::shared_ptr<Element> element = GetArgument(7))
	{
		randomDelay_maxSeconds = 60 * ToPositiveNumber(element->GetString());
	}

	uint32_t taskId = -1;
	uint32_t result_taskUpdate = era_Task_update(&taskId, windows, windowsProfileName_orNull, linux3, linux, mobile, randomDelay_maxSeconds, clients, params);
	if (result_taskUpdate != 0)
	{
		throw EraCMDException_FromApi(result_taskUpdate);
	}

	StringVector header(1);
	header[0] = L"task_id";
	StringVector prettyHeader(1);
	prettyHeader[0] = LoadStringFromResource(IDS_TASK_ID);
	std::vector<StringVector> rows;
	StringVector data(1);
	data[0] = std::to_wstring((long long)taskId);
	rows.push_back(data);
	TableReader_Const reader(header, prettyHeader, rows);
	out.Print(reader);
}

