/**
 * @file Command_Version.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_Version.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"
#include "../TableReader.h"
#include "../Version2.h"

void Command_Version::Execute(elementArray &elements, OutputHandler &out)
{
	StringVector header(2);
	header[0] = L"component";
	header[1] = L"version";

	StringVector prettyHeader(2);
	prettyHeader[0] = LoadStringFromResource(IDS_VERSION_COLNAME1);
	prettyHeader[1] = LoadStringFromResource(IDS_VERSION_COLNAME2);

	std::vector<StringVector> rows;
	std::shared_ptr<Element> whatArg = GetArgument(0);
	if (whatArg)
	{
		std::wstring what = whatArg->GetString();
		StringVector row(2);

		if (::_wcsicmp(what.c_str(), L"cmd") == 0)
		{
			row[0] = L"ERA CMD";
			row[1] = GetVersion_cmd();
		}
		else if (::_wcsicmp(what.c_str(), L"api") == 0)
		{
			row[0] = L"ERA API";
			row[1] = GetVersion_api();
		}
		else if (::_wcsicmp(what.c_str(), L"server") == 0)
		{
			row[0] = L"ERA server";
			row[1] = GetVersion_server();
		}
		else
		{
			throw EraCMDException(CMDERROR_INVALID_PARAMETER, what.c_str());
		}

		rows.push_back(std::move(row));
	}
	else
	{
		StringVector row(2);
		row[0] = L"ERA CMD";
		row[1] = GetVersion_cmd();
		rows.push_back(row);
		row[0] = L"ERA API";
		row[1] = GetVersion_api();
		rows.push_back(row);
		row[0] = L"ERA server";
		row[1] = GetVersion_server();
		rows.push_back(row);
	}

	TableReader_Const reader(header, prettyHeader, rows);
	out.Print(reader);
}

std::wstring Command_Version::GetVersion_cmd() const
{
	WCHAR wchVersionString[128] = L"";
	::GetVersionString(RA_CMD_VERSION, wchVersionString, sizeof(wchVersionString) / sizeof(wchVersionString[0]));
	return wchVersionString;
}

std::wstring Command_Version::GetVersion_api() const
{
	uint32_t Major;
	uint32_t Minor;
	uint32_t Build;
	(void)era_GetApiVersion(&Major, &Minor, &Build);

	std::wostringstream ss;
	ss << Major << '.' << Minor << '.' << Build;
	return ss.str();
}

std::wstring Command_Version::GetVersion_server() const
{
	Era_ServerInfo *serverInfo;

	uint32_t result_getServerInfo = era_GetServerInfo(&serverInfo);
	if (result_getServerInfo != 0)
	{
		throw EraCMDException_FromApi(result_getServerInfo);
	}
	scope_exit se_serverInfo(std::bind(era_ServerInfo_delete, serverInfo));

	std::wostringstream ss;
	ss << era_ServerInfo_GetMajorVersion(serverInfo) << '.' << era_ServerInfo_GetMinorVersion(serverInfo) << '.' << era_ServerInfo_GetBuildVersion(serverInfo);

	return ss.str();
}

