/**
 * @file stringUtils.h
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#pragma once

std::string WstringToString(const std::wstring &str);

uint32_t WstringToUint(const std::wstring &str);
int32_t WstringToInt(const std::wstring &str);

uint64_t WstringToUInt64(const std::wstring &str);
int64_t WstringToInt64(const std::wstring &str);

std::wstring LoadStringFromResource(unsigned int stringId);
std::wstring LoadStringFromFileInResource(unsigned int stringId, const std::wstring &resourceType);

int ReplaceAllSubstrings(std::wstring &str, const std::wstring &replaceFrom, const std::wstring &replaceTo);

std::wstring GetFirstSubstringBeforeDelimiter(std::wstring &src, const std::wstring &delimiter, bool removeFromSrc);

void SplitToWords(const std::wstring &src, WCHAR wordDelimiter, std::vector<std::wstring> &outResult, bool allowEmptyWords=false);

void WrapString(const std::wstring &src, size_t screenWidth, std::vector<std::wstring> &outResult);
std::wstring WrapString(const std::wstring &str, size_t screenWidth, bool addNewLineEvenIfSizeMatches);

FILETIME ToFileTime(const std::wstring &str);
bool ToBool(const std::wstring &str);
unsigned int ToPositiveNumber(const std::wstring &str);
int ParseHost(const std::wstring &host, std::wstring &server, uint16_t &port);

class ItemAndDescriptionFormatter
{
public:
	ItemAndDescriptionFormatter() {}
	void Add(const std::wstring &item, const std::wstring &description) { data.push_back(std::make_pair(item, description)); }
	std::wstring FormatOutput(unsigned int maxLineWidth);
	void SortByItem();
private:
	std::vector<std::pair<std::wstring, std::wstring> > data;
};
